@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2002 raf <raf@raf.org>
+* Copyright (C) 1999-2002, 2004, 2010, 2020-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-* or visit http://www.gnu.org/copyleft/gpl.html
+* along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20020916 raf <raf@raf.org>
+* 20230313 raf <raf@raf.org>
 */
 
 /*
@@ -58,6 +56,7 @@ I<libslack(err)> - message/error/debug/verbosity/alert messaging module
     void valertsys(int priority, const char *format, va_list args);
     int set_errno(int errnum);
     void *set_errnull(int errnum);
+    void (*(set_errnullf)(int errnum))();
 
     #define debug(args)
     #define vdebug(args)
@@ -67,14 +66,14 @@ I<libslack(err)> - message/error/debug/verbosity/alert messaging module
 
 =head1 DESCRIPTION
 
-This module works with the L<prog(3)|prog(3)> and L<msg(3)|msg(3)> modules
-to provide functions for emitting various types of message with simple call
-syntax and flexible behaviour. The message types catered for are: normal,
-verbose, debug, error, fatal error, dump and alert messages. All messages
-are created and sent with I<printf(3)>-like syntax. The destinations for
-these messages are configurable by the client. Calling I<prog_init(3)>
-causes normal and verbose messages to be sent to standard output; debug,
-error, fatal error, dump and alert messages to be sent to standard error.
+This module works with the I<prog(3)> and I<msg(3)> modules to provide
+functions for emitting various types of message with simple call syntax and
+flexible behaviour. The message types catered for are: normal, verbose,
+debug, error, fatal error, dump and alert messages. All messages are created
+and sent with I<printf(3)>-like syntax. The destinations for these messages
+are configurable by the client. Calling I<prog_init(3)> causes normal and
+verbose messages to be sent to standard output; and debug, error, fatal
+error, dump and alert messages to be sent to standard error.
 
 Calls to I<prog_set_out(3)>, I<prog_out_fd(3)>, I<prog_out_stdout(3)>,
 I<prog_out_file(3)>, I<prog_out_syslog(3)> and I<prog_out_none(3)> cause
@@ -98,8 +97,8 @@ destination.
 Calls to the generic functions I<prog_set_out(3)>, I<prog_set_err(3)>,
 I<prog_set_dbg(3)> and I<prog_set_alert(3)> cause their respective message
 types to be sent to the specified destination or destinations (multiplexing
-messages is only possible via these functions). See L<msg(3)|msg(3)> for
-more details.
+messages is only possible via these functions). See I<msg(3)> for more
+details.
 
 =over 4
 
@@ -107,6 +106,14 @@ more details.
 
 */
 
+#ifndef _BSD_SOURCE
+#define _BSD_SOURCE /* For snprintf() on OpenBSD-4.7 */
+#endif
+
+#ifndef _DEFAULT_SOURCE
+#define _DEFAULT_SOURCE /* New name for _BSD_SOURCE */
+#endif
+
 #include "config.h"
 #include "std.h"
 
@@ -125,14 +132,14 @@ more details.
 =item C<void msg(const char *format, ...)>
 
 Outputs a message to the program's normal message destination. C<format> is
-a I<printf(3)>-like format string and processes any remaining arguments in
+a I<printf(3)>-like format string which processes any remaining arguments in
 the same way as I<printf(3)>.
 
 B<Warning: Do not under any circumstances ever pass a non-literal string as
 the format argument unless you know exactly how many conversions will take
 place. Being careless with this is a very good way to build potential
-security holes into your programs. The same is true for all functions that
-take a printf()-like format string as an argument.>
+security vulnerabilities into your programs. The same is true for all
+functions that take a printf()-like format string as an argument.>
 
     msg(buf);       // EVIL
     msg("%s", buf); // GOOD
@@ -172,10 +179,10 @@ void vmsg(const char *format, va_list args)
 Outputs a verbose message to the program's normal message destination if
 C<level> is less than or equal to the program's current verbosity level. If
 the program's name has been supplied using I<prog_set_name(3)>, the message
-will be preceeded by the name, a colon and a space. The message is also
-preceeded by as many spaces as the message level. This indents messages
+will be preceded by the name, a colon, and a space. The message is also
+preceded by as many spaces as the message level. This indents messages
 according to their verbosity. C<format> is a I<printf(3)>-like format string
-and processes any remaining arguments in the same way as I<printf(3)>. The
+which processes any remaining arguments in the same way as I<printf(3)>. The
 message is followed by a newline.
 
 =cut
@@ -227,16 +234,19 @@ C<level> satisfies the program's current debug level. The debug level is
 broken into two components. The low byte specifies the level. The next three
 bytes specify a section within which the level applies. Debug messages with
 a section value whose bits overlap those of the program's current debug
-section and with a level that is less than or equal to the program's current
-debug level are emitted. As a convenience, if the program's current debug
-section is zero, debug messages with a sufficiently small level are emitted
-regardless of the message section. See I<prog_set_debug_level(3)> for
-examples. If the program's name has been supplied using I<prog_set_name(3)>,
-the message will be preceeded by the name, a colon and a space. The message
-is also preceeded by as many spaces as the debug level. This indents debug
-messages according to their debug level. C<format> is a I<printf(3)>-like
-format string and processes any remaining arguments in the same way as
-I<printf(3)>. The message is followed by a newline.
+section, and with a level that is less than or equal to the program's
+current debug level, are emitted. As a convenience, if the program's current
+debug section is zero, debug messages with a sufficiently small level are
+emitted, regardless of the message section. See I<prog_set_debug_level(3)>
+for examples. If the program's name has been supplied using
+I<prog_set_name(3)>, the message will be preceded by the name, a colon, and
+a space. This is followed by the string C<"debug:">. If C<level> specifies a
+non-zero section, it is included in the debug message, after a space and
+surrounded by square brackets. This is followed by as many spaces as the
+debug level. This indents debug messages according to their debug level.
+C<format> is a I<printf(3)>-like format string which processes any remaining
+arguments in the same way as I<printf(3)>. The message is followed by a
+newline.
 
 =cut
 
@@ -245,7 +255,7 @@ I<printf(3)>. The message is followed by a newline.
 static int debug_level_match(size_t level)
 {
 	size_t debug_level, debug_section, section;
-	
+
 	debug_level = prog_debug_level();
 	debug_section = debug_level & 0xffffff00;
 	debug_level &= 0x000000ff;
@@ -285,7 +295,7 @@ void vdebugf(size_t level, const char *format, va_list args)
 		vsnprintf(mesg, MSG_SIZE, format, args);
 
 		if (level & 0xffffff00)
-			snprintf(prefix, 32, " [%d]", (level & 0xffffff00) >> 8);
+			snprintf(prefix, 32, " [%d]", (int)((level & 0xffffff00) >> 8));
 
 		if (prog_name())
 			msg_out(prog_dbg(), "%s: debug:%s%*s%s\n", prog_name(), prefix, level & 0xff, "", mesg);
@@ -300,9 +310,10 @@ void vdebugf(size_t level, const char *format, va_list args)
 
 Outputs an error message to the program's error message destination. If the
 program's name has been supplied using I<prog_set_name(3)>, the message will
-be preceeded by the name, a colon and a space. C<format> is a
-I<printf(3)>-like format string and processes any remaining arguments in the
-same way as I<printf(3)>. The message is followed by a newline. Returns -1.
+be preceded by the name, a colon, and a space. C<format> is a
+I<printf(3)>-like format string which processes any remaining arguments in
+the same way as I<printf(3)>. The message is followed by a newline. Returns
+-1.
 
 =cut
 
@@ -346,14 +357,14 @@ int verror(const char *format, va_list args)
 
 =item C<void fatal(const char *format, ...)>
 
-Outputs an error message to the program's error message destination and then
-calls I<exit(3)> with a return code of C<EXIT_FAILURE>. If the program's
-name was supplied using I<prog_set_name(3)>, the message will be preceeded
-by the name, a colon and a space. This is followed by the string C<"fatal:
-">. C<format> is a I<printf(3)>-like format string and processes any
-remaining arguments in the same way as I<printf(3)>. The message is followed
-by a newline. B<Note:> Never use this in a library. Only an application can
-decide which errors are fatal.
+Outputs an error message to the program's error message destination, and
+then calls I<exit(3)> with a return code of C<EXIT_FAILURE>. If the
+program's name was supplied using I<prog_set_name(3)>, the message will be
+preceded by the name, a colon, and a space. This is followed by the string
+C<"fatal: ">. C<format> is a I<printf(3)>-like format string which processes
+any remaining arguments in the same way as I<printf(3)>. The message is
+followed by a newline. B<Note:> Never use this in a library. Only an
+application can decide which errors are fatal.
 
 =cut
 
@@ -392,10 +403,10 @@ void vfatal(const char *format, va_list args)
 
 Outputs an error message to the program's error message destination and then
 calls I<abort(3)>. If the program's name was supplied using
-I<prog_set_name(3)>, the message will be preceeded by the name, a colon and
+I<prog_set_name(3)>, the message will be preceded by the name, a colon, and
 a space. This is followed by the string C<"dump: ">. C<format> is a
-I<printf(3)>-like format string and processes any remaining arguments in the
-same way as I<printf(3)>. The message is followed by a newline. B<Note:>
+I<printf(3)>-like format string which processes any remaining arguments in
+the same way as I<printf(3)>. The message is followed by a newline. B<Note:>
 Never use this in a library. Only an application can decide which errors are
 fatal.
 
@@ -436,8 +447,8 @@ void vdump(const char *format, va_list args)
 
 Outputs an alert message of the given C<priority> to the program's alert
 message destination. If the program's name has been supplied using
-I<prog_set_name(3)>, the message will be preceeded by the name, a colon and
-a space. C<format> is a I<printf(3)>-like format string and processes any
+I<prog_set_name(3)>, the message will be preceded by the name, a colon, and
+a space. C<format> is a I<printf(3)>-like format string which processes any
 remaining arguments in the same way as I<printf(3)>. The message is followed
 by a newline. Note that this only works when the program's alert message
 destination is a simple syslog destination. If the alert message destination
@@ -499,8 +510,8 @@ void valert(int priority, const char *format, va_list args)
 =item C<void debugsysf(size_t level, const char *format, ...)>
 
 Equivalent to I<debugf(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -544,8 +555,8 @@ void vdebugsysf(size_t level, const char *format, va_list args)
 =item C<int errorsys(const char *format, ...)>
 
 Equivalent to I<error(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -584,8 +595,8 @@ int verrorsys(const char *format, va_list args)
 =item C<void fatalsys(const char *format, ...)>
 
 Equivalent to I<fatal(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -623,8 +634,8 @@ void vfatalsys(const char *format, va_list args)
 =item C<void dumpsys(const char *format, ...)>
 
 Equivalent to I<dump(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -662,8 +673,8 @@ void vdumpsys(const char *format, va_list args)
 =item C<void alertsys(int priority, const char *format, ...)>
 
 Equivalent to I<alert(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -730,6 +741,24 @@ void *(set_errnull)(int errnum)
 
 /*
 
+=item C<void (*set_errnullf(int errnum))()>
+
+Sets C<errno> to C<errnum> and returns C<null> as a function pointer.
+This is useful because I<ISO C> doesn't like casting normal pointers
+into function pointers.
+
+=cut
+
+*/
+
+void (*(set_errnullf)(int errnum))()
+{
+	errno = errnum;
+	return NULL;
+}
+
+/*
+
 =item C< #define debug(args)>
 
 Calls I<debugf(3)> unless C<NDEBUG> is defined. C<args> must be supplied
@@ -737,6 +766,10 @@ with extra parentheses. e.g.
 
     debug((1, "rc=%d", rc))
 
+Note that the I<debug()> macro always generates its own semicolon. This used
+to only be the case when C<NDEBUG> was not defined, but it is now
+consistent.
+
 =item C< #define vdebug(args)>
 
 Calls I<vdebugf(3)> unless C<NDEBUG> is defined. C<args> must be supplied
@@ -744,6 +777,10 @@ with extra parentheses. e.g.
 
     vdebug((1, format, args))
 
+Note that the I<vdebug()> macro always generates its own semicolon. This
+used to only be the case when C<NDEBUG> was not defined, but it is now
+consistent.
+
 =item C< #define debugsys(args)>
 
 Calls I<debugsysf(3)> unless C<NDEBUG> is defined. C<args> must be supplied
@@ -751,6 +788,10 @@ with extra parentheses. e.g.
 
     debugsys((1, "fd=%d", fd))
 
+Note that the I<debugsys()> macro always generates its own semicolon. This
+used to only be the case when C<NDEBUG> was not defined, but it is now
+consistent.
+
 =item C< #define vdebugsys(args)>
 
 Calls I<vdebugsysf(3)> unless C<NDEBUG> is defined. C<args> must be supplied
@@ -758,6 +799,10 @@ with extra parentheses. e.g.
 
     vdebugsys((1, format, args))
 
+Note that the I<vdebugsys()> macro always generates its own semicolon. This
+used to only be the case when C<NDEBUG> was not defined, but it is now
+consistent.
+
 =item C< #define check(cond, mesg)>
 
 Like I<assert(3)> but includes a string argument, C<mesg>, for including an
@@ -765,7 +810,7 @@ explanation of the condition tested and then calls I<dump(3)> to terminate
 the program. This means the message will be sent to the right place(s)
 rather than just to C<stderr>. B<Note:> Like I<assert(3)>, this function is
 largely useless. It should never be left in production code (because it's
-rude) so you need to write code to handle error conditions properly anyway.
+rude), so you need to write code to handle error conditions properly anyway.
 You might as well not bother using I<assert(3)> or I<check(3)> in the first
 place.
 
@@ -773,18 +818,46 @@ place.
 
 =head1 MT-Level
 
-MT-Safe
+I<MT-Safe>
+
+=head1 EXAMPLES
+
+Send a range of messages to default locations:
+
+    #include <slack/std.h>
+    #include <slack/prog.h>
+    #include <slack/err.h>
+
+    int main(int ac, char **av)
+    {
+        prog_init();
+        prog_set_debug_level(1);
+        prog_set_verbosity_level(1);
+
+        msg("This is a %s\n", "message");
+        verbose(0, "This is a %s message (level %d)", "verbose", 0);
+        verbose(1, "This is a %s message (level %d)", "verbose", 1);
+        verbose(2, "This is a %s message (level %d)", "verbose", 2);
+        debug((0, "This is a %s message (level %d)", "debug", 0))
+        debug((1, "This is a %s message (level %d)", "debug", 1))
+        debug((2, "This is a %s message (level %d)", "debug", 2))
+        alert(LOG_ERR, "This is an %s message", "alert");
+        error("This is an %s message", "error");
+        fatal("This is a %s message", "fatal error");
+
+        return EXIT_SUCCESS;
+    }
 
 =head1 SEE ALSO
 
-L<libslack(3)|libslack(3)>,
-L<msg(3)|msg(3)>,
-L<prog(3)|prog(3)>,
-L<printf(3)|printf(3)>
+I<libslack(3)>,
+I<msg(3)>,
+I<prog(3)>,
+I<printf(3)>
 
 =head1 AUTHOR
 
-20020916 raf <raf@raf.org>
+20230313 raf <raf@raf.org>
 
 =cut
 
@@ -799,6 +872,9 @@ L<printf(3)|printf(3)>
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/resource.h>
+
+#include "str.h"
 
 int verify(int test, const char *name, const char *result)
 {
@@ -841,6 +917,11 @@ int verifysys(int test, const char *name, const char *result, int err)
 	return verify(test, name, buf);
 }
 
+int msg_filter_prefix(void **mesgp, const void *mesg, size_t mesglen)
+{
+	return asprintf((char **)mesgp, "[%d] %.*s", 12345, (int)mesglen, (char *)mesg);
+}
+
 int main(int ac, char **av)
 {
 	const char * const out = "err.out";
@@ -852,6 +933,7 @@ int main(int ac, char **av)
 	char buf[BUFSIZ];
 	int rci;
 	void *rcp;
+	void (*rcfp)();
 
 	const char *results[12] =
 	{
@@ -907,6 +989,18 @@ int main(int ac, char **av)
 	error("errormsg");
 	errors += verify(4, err, results[3]);
 
+	/* Test filtered debug */
+
+	prog_dbg_file(dbg);
+	if (prog_dbg_push_filter(msg_filter_prefix) == -1)
+		++errors, printf("Test5: prog_dbg_push_filter() failed\n");
+	else
+	{
+		debugf(1, "filteredmsg");
+		errors += verify(6, dbg, "[12345] debug: filteredmsg");
+	}
+
+
 	/* Test fatal */
 
 	switch (pid = fork())
@@ -920,7 +1014,7 @@ int main(int ac, char **av)
 		case -1:
 		{
 			++errors;
-			printf("Test5: failed to perform test - fork() failed (%s)\n", strerror(errno));
+			printf("Test7: failed to perform test - fork() failed (%s)\n", strerror(errno));
 			break;
 		}
 
@@ -930,18 +1024,18 @@ int main(int ac, char **av)
 
 			if (waitpid(pid, &status, 0) == -1)
 			{
-				++errors, printf("Test5: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
+				++errors, printf("Test7: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
 				break;
 			}
 
 			if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_FAILURE)
-				++errors, printf("Test5: failed: %s %d\n",
+				++errors, printf("Test7: failed: %s %d (expected: exit code %d)\n",
 					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), EXIT_FAILURE);
 		}
 	}
 
-	errors += verify(5, err, results[4]);
+	errors += verify(7, err, results[4]);
 
 	/* Test dump */
 
@@ -958,50 +1052,59 @@ int main(int ac, char **av)
 		case -1:
 		{
 			++errors;
-			printf("Test6: failed to perform test - fork() failed (%s)\n", strerror(errno));
+			printf("Test8: failed to perform test - fork() failed (%s)\n", strerror(errno));
 			break;
 		}
 
 		default:
 		{
-			struct stat statbuf[1];
 			int status;
 
 			if (waitpid(pid, &status, 0) == -1)
 			{
-				++errors, printf("Test6: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
+				++errors, printf("Test8: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
 				break;
 			}
 
 			if (!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT)
-				++errors, printf("Test5: failed: %s %d\n",
-					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
-
-			if (stat(core, statbuf) == -1 && errno == ENOENT &&
-				stat(core2, statbuf) == -1 && errno == ENOENT)
-				++errors, printf("Test6: failed: no core file produced (ulimit?)\n");
-			else
 			{
-				unlink(core);
-				unlink(core2);
+				++errors, printf("Test8: failed: %s %d (expected: received signal %d)\n",
+					WIFSIGNALED(status) ? "received signal" : "exit code",
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), SIGABRT);
 			}
+
+			/* Check for core dump, but ulimit might prevent them */
+
+#ifdef WCOREDUMP
+			if (WIFSIGNALED(status) && !WCOREDUMP(status))
+			{
+				struct rlimit limit[1] = {{ 0, 0 }};
+
+				if (getrlimit(RLIMIT_CORE, limit) != -1 && limit->rlim_cur != 0)
+					printf("Warning: dump() produced no core file (even though ulimit -c %d)\n", (int)limit->rlim_cur);
+			}
+#endif
+
+			/* Core files might be anywhere these days, but they might be here */
+
+			unlink(core);
+			unlink(core2);
 		}
 	}
 
-	errors += verify(6, err, results[5]);
+	errors += verify(8, err, results[5]);
 
 	/* Test debugsys, errorsys */
 
 	prog_dbg_file(dbg);
 	set_errno(EPERM);
 	debugsysf(1, "debugsysmsg");
-	errors += verifysys(7, dbg, results[6], EPERM);
+	errors += verifysys(9, dbg, results[6], EPERM);
 
 	prog_err_file(err);
 	set_errno(ENOENT);
 	errorsys("errorsysmsg");
-	errors += verifysys(8, err, results[7], ENOENT);
+	errors += verifysys(10, err, results[7], ENOENT);
 
 	/* Test fatalsys */
 
@@ -1014,98 +1117,6 @@ int main(int ac, char **av)
 			fatalsys("fatalsysmsg");
 		}
 
-		case -1:
-		{
-			++errors;
-			printf("Test9: failed to perform test - fork() failed (%s)\n", strerror(errno));
-			break;
-		}
-
-		default:
-		{
-			int status;
-
-			if (waitpid(pid, &status, 0) == -1)
-			{
-				++errors;
-				printf("Test9: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
-				break;
-			}
-
-			if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_FAILURE)
-				++errors, printf("Test5: failed: %s %d\n",
-					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
-		}
-	}
-
-	errors += verifysys(9, err, results[8], EPERM);
-
-	/* Test dumpsys */
-
-	switch (pid = fork())
-	{
-		case 0:
-		{
-			prog_err_file(err);
-			unlink(core);
-			unlink(core2);
-			set_errno(ENOENT);
-			dumpsys("dumpsysmsg");
-		}
-
-		case -1:
-		{
-			++errors;
-			printf("Test10: failed to perform test - fork() failed (%s)\n", strerror(errno));
-			break;
-		}
-
-		default:
-		{
-			struct stat statbuf[1];
-			int status;
-
-			if (waitpid(pid, &status, 0) == -1)
-			{
-				++errors;
-				printf("Test10: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
-				break;
-			}
-
-			if (!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT)
-				++errors, printf("Test5: failed: %s %d\n",
-					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
-
-			if (stat(core, statbuf) == -1 && errno == ENOENT &&
-				stat(core2, statbuf) == -1 && errno == ENOENT)
-				++errors, printf("Test10: failed: no core file produced (ulimit?)\n");
-			else
-			{
-				unlink(core);
-				unlink(core2);
-			}
-		}
-	}
-
-	errors += verifysys(10, err, results[9], ENOENT);
-
-	/* Test check true */
-
-	switch (pid = fork())
-	{
-		case 0:
-		{
-			int i = 1;
-			prog_err_file(err);
-			unlink(core);
-			unlink(core2);
-			set_errno(ENOENT);
-			check(i == 1, "checkmsg");
-			_exit(EXIT_SUCCESS);
-		}
-
 		case -1:
 		{
 			++errors;
@@ -1124,10 +1135,109 @@ int main(int ac, char **av)
 				break;
 			}
 
-			if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_SUCCESS)
-				++errors, printf("Test5: failed: %s %d\n",
+			if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_FAILURE)
+				++errors, printf("Test11: failed: %s %d (expected: exit code %d)\n",
 					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), EXIT_FAILURE);
+		}
+	}
+
+	errors += verifysys(11, err, results[8], EPERM);
+
+	/* Test dumpsys */
+
+	switch (pid = fork())
+	{
+		case 0:
+		{
+			prog_err_file(err);
+			unlink(core);
+			unlink(core2);
+			set_errno(ENOENT);
+			dumpsys("dumpsysmsg");
+		}
+
+		case -1:
+		{
+			++errors;
+			printf("Test12: failed to perform test - fork() failed (%s)\n", strerror(errno));
+			break;
+		}
+
+		default:
+		{
+			int status;
+
+			if (waitpid(pid, &status, 0) == -1)
+			{
+				++errors;
+				printf("Test12: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
+				break;
+			}
+
+			if (!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT)
+				++errors, printf("Test12: failed: %s %d (expected: received signal %d)\n",
+					WIFSIGNALED(status) ? "received signal" : "exit code",
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), SIGABRT);
+
+			/* Check for core dump, but ulimit might prevent them */
+
+#ifdef WCOREDUMP
+			if (WIFSIGNALED(status) && !WCOREDUMP(status))
+			{
+				struct rlimit limit[1] = {{ 0, 0 }};
+
+				if (getrlimit(RLIMIT_CORE, limit) != -1 && limit->rlim_cur != 0)
+					printf("Warning: dump() produced no core file (even though ulimit -c %d)\n", (int)limit->rlim_cur);
+			}
+#endif
+
+			/* Core files might be anywhere these days, but they might be here */
+
+			unlink(core);
+			unlink(core2);
+		}
+	}
+
+	errors += verifysys(12, err, results[9], ENOENT);
+
+	/* Test check true */
+
+	switch (pid = fork())
+	{
+		case 0:
+		{
+			int i = 1;
+			prog_err_file(err);
+			unlink(core);
+			unlink(core2);
+			set_errno(ENOENT);
+			check(i == 1, "checkmsg");
+			_exit(EXIT_SUCCESS);
+		}
+
+		case -1:
+		{
+			++errors;
+			printf("Test13: failed to perform test - fork() failed (%s)\n", strerror(errno));
+			break;
+		}
+
+		default:
+		{
+			int status;
+
+			if (waitpid(pid, &status, 0) == -1)
+			{
+				++errors;
+				printf("Test13: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
+				break;
+			}
+
+			if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_SUCCESS)
+				++errors, printf("Test13: failed: %s %d (expected: exit code %d)\n",
+					WIFSIGNALED(status) ? "received signal" : "exit code",
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), EXIT_SUCCESS);
 		}
 	}
 
@@ -1149,40 +1259,47 @@ int main(int ac, char **av)
 		case -1:
 		{
 			++errors;
-			printf("Test12: failed to perform test - fork() failed (%s)\n", strerror(errno));
+			printf("Test14: failed to perform test - fork() failed (%s)\n", strerror(errno));
 			break;
 		}
 
 		default:
 		{
-			struct stat statbuf[1];
 			int status;
 
 			if (waitpid(pid, &status, 0) == -1)
 			{
 				++errors;
-				printf("Test12: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
+				printf("Test14: failed to wait for test - waitpid(%d) failed (%s)\n", (int)pid, strerror(errno));
 				break;
 			}
 
 			if (!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT)
-				++errors, printf("Test5: failed: %s %d\n",
+				++errors, printf("Test14: failed: %s %d (expected: received signal %d)\n",
 					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), SIGABRT);
 
-			if (stat(core, statbuf) == -1 && errno == ENOENT &&
-				stat(core2, statbuf) == -1 && errno == ENOENT)
-				++errors, printf("Test12: failed: no core file produced (ulimit?)\n");
-			else
+			/* Check for core dump, but ulimit might prevent them */
+
+#ifdef WCOREDUMP
+			if (WIFSIGNALED(status) && !WCOREDUMP(status))
 			{
-				unlink(core);
-				unlink(core2);
+				struct rlimit limit[1] = {{ 0, 0 }};
+
+				if (getrlimit(RLIMIT_CORE, limit) != -1 && limit->rlim_cur != 0)
+					printf("Warning: dump() produced no core file (even though ulimit -c %d)\n", (int)limit->rlim_cur);
 			}
+#endif
+
+			/* Core files might be anywhere these days, but they might be here */
+
+			unlink(core);
+			unlink(core2);
 		}
 	}
 
 	snprintf(buf, BUFSIZ, "dump: Internal Error: %s: %s [", "i == 0", "checkmsg");
-	errors += verify(12, err, buf);
+	errors += verify(14, err, buf);
 
 	/* Test debug sections */
 
@@ -1208,7 +1325,7 @@ int main(int ac, char **av)
     debugf(1, "global debugmsg");                  /* yes */
     debugf(4, "global debugmsg");                  /* no (level too high) */
 
-	errors += verify(13, dbg, results[10]);
+	errors += verify(15, dbg, results[10]);
 
 #undef LEXER_SECTION
 #undef PARSER_SECTION
@@ -1220,29 +1337,36 @@ int main(int ac, char **av)
 	alert(LOG_INFO, "alertmsg");
 	errno = 0;
 	alertsys(LOG_INFO, "alertsysmsg");
-	errors += verify(14, alertfile, results[11]);
+	errors += verify(16, alertfile, results[11]);
 
 	prog_out_none();
 	prog_err_none();
 	prog_dbg_none();
 	prog_alert_none();
 
-	/* Test set_errno() and set_errnull() */
+	/* Test set_errno() and set_errnull() and set_errnullf() */
 
 	errno = 0;
 	if ((rci = set_errno(EINVAL)) != -1)
-		++errors, printf("Test15: set_errno(EINVAL) failed (returned %d, not %d)\n", rci, -1);
+		++errors, printf("Test17: set_errno(EINVAL) failed (returned %d, not %d)\n", rci, -1);
 	else if (errno != EINVAL)
-		++errors, printf("Test15: set_errno(EINVAL) failed (errno = %s, not %s)\n", strerror(errno), strerror(EINVAL));
+		++errors, printf("Test17: set_errno(EINVAL) failed (errno = %s, not %s)\n", strerror(errno), strerror(EINVAL));
 
 	errno = 0;
 	if ((rcp = set_errnull(EINVAL)) != NULL)
-		++errors, printf("Test16: set_errnull(EINVAL) failed (returned %p, not %p)\n", rcp, (void *)NULL);
+		++errors, printf("Test18: set_errnull(EINVAL) failed (returned %p, not %p)\n", rcp, (void *)NULL);
 	else if (errno != EINVAL)
-		++errors, printf("Test16: set_errnull(EINVAL) failed (errno = %s, not %s)\n", strerror(errno), strerror(EINVAL));
+		++errors, printf("Test18: set_errnull(EINVAL) failed (errno = %s, not %s)\n", strerror(errno), strerror(EINVAL));
+
+	errno = 0;
+	if ((rcfp = set_errnullf(EINVAL)) != NULL)
+		/* ++errors, printf("Test19: set_errnullf(EINVAL) failed (returned %p, not %p)\n", rcfp, (void *)NULL); */
+		++errors, printf("Test19: set_errnullf(EINVAL) failed (returned something other than null)\n");
+	else if (errno != EINVAL)
+		++errors, printf("Test19: set_errnullf(EINVAL) failed (errno = %s, not %s)\n", strerror(errno), strerror(EINVAL));
 
 	if (errors)
-		printf("%d/16 tests failed\n", errors);
+		printf("%d/19 tests failed\n", errors);
 	else
 		printf("All tests passed\n");
 
