@@ -1,7 +1,7 @@
 /*
 # libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2001 raf <raf@raf.org>
+* Copyright (C) 1999-2002, 2004, 2010, 2020-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-* or visit http://www.gnu.org/copyleft/gpl.html
+* along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20011109 raf <raf@raf.org>
+* 20230313 raf <raf@raf.org>
 */
 
 /*
@@ -29,6 +27,7 @@ I<libslack(fio)> - fifo and file control module and some I/O
 
 =head1 SYNOPSIS
 
+    #include <slack/std.h>
     #include <slack/fio.h>
 
     char *fgetline(char *line, size_t size, FILE *stream);
@@ -39,6 +38,8 @@ I<libslack(fio)> - fifo and file control module and some I/O
     int nap(long sec, long usec);
     int fcntl_set_flag(int fd, int flag);
     int fcntl_clear_flag(int fd, int flag);
+    int fcntl_set_fdflag(int fd, int flag);
+    int fcntl_clear_fdflag(int fd, int flag);
     int fcntl_lock(int fd, int cmd, int type, int whence, int start, int len);
     int nonblock_set(int fd, int arg);
     int nonblock_on(int fd);
@@ -61,10 +62,19 @@ shorthand functions for manipulating file flags and locks.
 */
 
 #include "config.h"
+
+#ifndef NO_POSIX_SOURCE
+#define NO_POSIX_SOURCE /* For ETIMEDOUT, EADDRINUSE, EOPNOTSUPP on FreeBSD-8.0 */
+#endif
+
 #include "std.h"
 
 #include <fcntl.h>
 
+#include <sys/types.h>
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 #include <sys/time.h>
 #include <sys/stat.h>
 
@@ -86,16 +96,23 @@ void (funlockfile)(FILE *stream);
 
 =item C<char *fgetline(char *line, size_t size, FILE *stream)>
 
-Similar to I<fgets(3)> except that it recognises UNIX ("\n"), DOS ("\r\n")
-and Macintosh ("\r") line endings (even different line ending in the same
-file). Reads in at most C<size - 1> characters from C<stream> and stores
-them into the buffer pointed to by C<line>. Reading stops after an C<EOF> or
-the end of the line. If the end of the line was read, a newline is stored
-into the buffer. A C<null> is stored after the last character in the buffer.
-On success, returns C<line>. On error, or when the end of file occurs while
-no characters have been read, returns C<null>. Calls to this function can be
-mixed with calls to other input functions from the I<stdio> library for the
-same input stream.
+Similar to I<fgets(3)> except that it recognises UNIX (C<"\n">), DOS/Windows
+(C<"\r\n">) and old Macintosh (C<"\r">) line endings (even different line
+endings in the same file). Reads bytes from C<stream> and stores them in the
+buffer pointed to by C<line>. Reading stops after an C<EOF>, or the end of
+the line is reached, or when C<size - 1> bytes have been stored. If the end
+of the line was reached, it is stored as a C<"\n"> byte. A C<nul> byte is
+stored after the last byte in the buffer. On success, returns C<line>. On
+error, or when the end of file occurs while no bytes have been read, returns
+C<null>. Note that even when C<null> is returned, C<line> is modified and
+will always be C<nul>-terminated. So it is safe to examine C<line> even when
+this function returns C<null>. Calls to this function can be mixed with
+calls to other input functions from the I<stdio> library for the same input
+stream. This is a drop-in replacement for I<fgets(3)>.
+
+    char line[BUFSIZ];
+    while (fgetline(line, BUFSIZ, stdin))
+        printf("%s", line);
 
 =cut
 
@@ -116,7 +133,7 @@ char *fgetline(char *line, size_t size, FILE *stream)
 
 =item C<char *fgetline_unlocked(char *line, size_t size, FILE *stream)>
 
-Equivalent to I<fgetline()> except that C<stream> is not locked.
+Equivalent to I<fgetline(3)> except that C<stream> is not locked.
 
 =cut
 
@@ -152,17 +169,11 @@ char *fgetline_unlocked(char *line, size_t size, FILE *stream)
 			*s++ = c;
 	}
 
-	if (s < end && c == EOF)
-	{
-		if (ferror(stream))
-			return NULL;
-
-		if (s == line && feof(stream))
-			return NULL;
-	}
-
 	*s = '\0';
 
+	if (c == EOF && (s == line || ferror(stream)))
+		return NULL;
+
 	return line;
 }
 
@@ -179,8 +190,8 @@ I<alarm(3)> and C<SIGALRM> signals (best avoided). On success, returns C<0>.
 On error, returns C<-1> with C<errno> set appropriately (C<ETIMEDOUT> if it
 timed out, otherwise set by I<select(2)>). Usage:
 
-	if (read_timeout(fd, 5, 0) == -1 || (bytes = read(fd, buf, count)) == -1)
-		return -1;
+    if (read_timeout(fd, 5, 0) == -1 || (bytes = read(fd, buf, count)) == -1)
+        return -1;
 
 =cut
 
@@ -224,8 +235,8 @@ I<alarm(3)> and C<SIGALRM> signals (best avoided). On success, returns C<0>.
 On error, returns C<-1> with C<errno> set appropriately (C<ETIMEDOUT> if it
 timed out, otherwise set by I<select(2)>). Usage:
 
-	if (write_timeout(fd, 5, 0) == -1 || (bytes = write(fd, buf, count)) == -1)
-		return -1;
+    if (write_timeout(fd, 5, 0) == -1 || (bytes = write(fd, buf, count)) == -1)
+        return -1;
 
 =cut
 
@@ -269,6 +280,15 @@ and/or has urgent data available (C<X_OK>). On error, returns C<-1> with
 C<errno> set appropriately (C<ETIMEDOUT> if it timed out, otherwise set by
 I<select(2)>).
 
+    if ((mask = rw_timeout(fd, 5, 0)) == -1)
+        return -1;
+
+    if ((mask & W_OK) && (bytes = write(fd, buf, count)) == -1)
+        return -1;
+
+    if ((mask & R_OK) && (bytes = read(fd, buf, count)) == -1)
+        return -1;
+
 =cut
 
 */
@@ -316,8 +336,13 @@ int rw_timeout(int fd, long sec, long usec)
 =item C<int nap(long sec, long usec)>
 
 Puts the process to sleep for C<sec> seconds and C<usec> microseconds. Note,
-however, that many systems' timers only have 10ms resolution. On success,
-returns C<0>. On error, returns C<-1> with C<errno> set appropriately.
+however, that many systems' timers only have 10ms resolution. This uses
+I<select(3)> to ensure that I<alarm(3)> and C<SIGALRM> signals are not used
+(best avoided). On success, returns C<0>. On error, returns C<-1> with
+C<errno> set appropriately.
+
+    nap(1, 500000); // Sleep for 1.5 seconds
+    nap(0, 100000); // Sleep for 0.1 seconds
 
 =cut
 
@@ -340,10 +365,12 @@ int nap(long sec, long usec)
 
 =item C<int fcntl_set_flag(int fd, int flag)>
 
-Shorthand for setting the file flag, C<flag>, on the file descriptor, C<fd>,
-using I<fcntl(2)>. All other flags are unaffected. On success, returns C<0>.
-On error, returns C<-1> with C<errno> set by I<fcntl(2)> with C<F_GETFL> or
-C<F_SETFL> as the command.
+Shorthand for setting the file status flag, C<flag>, on the file descriptor,
+C<fd>, using I<fcntl(2)>. All other file status flags are unaffected. On
+success, returns C<0>. On error, returns C<-1> with C<errno> set by
+I<fcntl(2)> with C<F_GETFL> or C<F_SETFL> as the command. Example file
+status flags are C<O_APPEND>, C<O_ASYNC>, C<O_DIRECT>, C<O_NOATIME>, and
+C<O_NONBLOCK> depending on the system.
 
 =cut
 
@@ -363,10 +390,12 @@ int fcntl_set_flag(int fd, int flag)
 
 =item C<int fcntl_clear_flag(int fd, int flag)>
 
-Shorthand for clearing the file flag, C<flag>, from the file descriptor,
-C<fd>, using I<fcntl(2)>. All other flags are unaffected. On success,
-returns C<0>. On error, returns C<-1> with C<errno> set by I<fcntl(2)> with
-C<F_GETFL> or C<F_SETFL> as the command.
+Shorthand for clearing the file status flag, C<flag>, from the file
+descriptor, C<fd>, using I<fcntl(2)>. All other file status flags are
+unaffected. On success, returns C<0>. On error, returns C<-1> with C<errno>
+set by I<fcntl(2)> with C<F_GETFL> or C<F_SETFL> as the command. Example
+file status flags are C<O_APPEND>, C<O_ASYNC>, C<O_DIRECT>, C<O_NOATIME>,
+and C<O_NONBLOCK> depending on the system.
 
 =cut
 
@@ -384,6 +413,54 @@ int fcntl_clear_flag(int fd, int flag)
 
 /*
 
+=item C<int fcntl_set_fdflag(int fd, int flag)>
+
+Shorthand for setting the file descriptor flag, C<flag>, on the file
+descriptor, C<fd>, using I<fcntl(2)>. All other file descriptor flags are
+unaffected. On success, returns C<0>. On error, returns C<-1> with C<errno>
+set by I<fcntl(2)> with C<F_GETFD> or C<F_SETFD> as the command. The only
+file descriptor flag at time of writing is C<FD_CLOEXEC>.
+
+=cut
+
+*/
+
+int fcntl_set_fdflag(int fd, int flag)
+{
+	int flags;
+
+	if ((flags = fcntl(fd, F_GETFD, 0)) == -1)
+		return -1;
+
+	return fcntl(fd, F_SETFD, flags | flag);
+}
+
+/*
+
+=item C<int fcntl_clear_fdflag(int fd, int flag)>
+
+Shorthand for clearing the file descriptor flag, C<flag>, from the file
+descriptor, C<fd>, using I<fcntl(2)>. All other file descriptor flags are
+unaffected. On success, returns C<0>. On error, returns C<-1> with C<errno>
+set by I<fcntl(2)> with C<F_GETFD> or C<F_SETFD> as the command. The only
+file descriptor flag at time of writing is C<FD_CLOEXEC>.
+
+=cut
+
+*/
+
+int fcntl_clear_fdflag(int fd, int flag)
+{
+	int flags;
+
+	if ((flags = fcntl(fd, F_GETFL, 0)) == -1)
+		return -1;
+
+	return fcntl(fd, F_SETFL, flags & ~flag);
+}
+
+/*
+
 =item C<int fcntl_lock(int fd, int cmd, int type, int whence, int start, int len)>
 
 Shorthand for performing discretionary file locking operations on the file
@@ -392,6 +469,9 @@ I<fcntl(2)>. C<type>, C<whence>, C<start> and C<len> are used to fill a
 I<flock> structure which is passed to I<fcntl(2)>. Returns the same as
 I<fcntl(2)> with C<cmd> as the command.
 
+    if (fcntl_lock(fd, F_SETLK, F_WRLCK, SEEK_SET, 0, 0) == -1)
+        return -1;
+
 =cut
 
 */
@@ -496,8 +576,8 @@ int fifo_exists(const char *path, int prepare)
 =item C<int fifo_has_reader(const char *path, int prepare)>
 
 Determines whether or not C<path> refers to a fifo that is being read by
-another process. If C<path> does not exist or does not refer to a fifo or if
-the fifo can't be opened for non-blocking I<write(2)>, returns C<0>. If
+another process. If C<path> does not exist, or does not refer to a fifo, or
+if the fifo can't be opened for non-blocking I<write(2)>, returns C<0>. If
 C<prepare> is non-zero, and path refers to a non-fifo, it will be unlinked.
 On error, returns C<-1> with C<errno> set by I<stat(2)> or I<open(2)>.
 
@@ -540,18 +620,30 @@ int fifo_has_reader(const char *path, int prepare)
 =item C<int fifo_open(const char *path, mode_t mode, int lock, int *writefd)>
 
 Creates a fifo named C<path> with creation mode C<mode> for reading. If
-C<path> already exists, is a fifo and has a reader process, returns C<-1>
+C<path> already exists, is a fifo, and has a reader process, returns C<-1>
 with C<errno> set to C<EADDRINUSE>. If the fifo is created (or an existing
 one can be reused), two file descriptors are opened to the fifo. A read
 descriptor and a write descriptor. On success, returns the read descriptor.
 The write descriptor only exists to ensure that there is always at least one
 writer process for the fifo. This allows a I<read(2)> on the read descriptor
 to block until another process writes to the fifo rather than returning an
-C<EOF> condition. This is done in a POSIX compliant way. If C<lock> is
+C<EOF> condition. This is done in a I<POSIX>-compliant way. If C<lock> is
 non-zero, the fifo is exclusively locked. If C<writefd> is not C<null>, the
 write descriptor is stored there. On error, returns C<-1> with C<errno> set
 by I<stat(2)>, I<open(2)>, I<mkfifo(2)>, I<fstat(2)> or I<fcntl(2)>.
 
+    char *fifopath = "/tmp/fifo";
+    int fd, wfd;
+
+    if ((fd = fifo_open(fifopath, S_IRUSR | S_IWUSR | S_IWGRP | S_IWOTH, 1, &wfd)) == -1)
+        return -1;
+
+    // Read from fd...
+
+    close(fd);
+    close(wfd);
+    unlink(fifopath);
+
 =cut
 
 */
@@ -628,10 +720,14 @@ int fifo_open(const char *path, mode_t mode, int lock, int *writefd)
 	** deciding that there's no reader and opening this fifo
 	** at the same time.
 	**
-	** Note: some systems (like Mac OS X) can't lock fifos :(
+	** Note: some systems (e.g. FreeBSD, Mac OS X) can't lock fifos :(
 	*/
 
-	if (lock && fcntl_lock(wfd, F_SETLK, F_WRLCK, SEEK_SET, 0, 0) == -1 && errno != EOPNOTSUPP)
+/* On MacOSX-10.6 these are different numbers */
+#ifndef ENOTSUP
+#define ENOTSUP EOPNOTSUPP
+#endif
+	if (lock && fcntl_lock(wfd, F_SETLK, F_WRLCK, SEEK_SET, 0, 0) == -1 && errno != EOPNOTSUPP && errno != ENOTSUP && errno != EBADF)
 	{
 		if (mine)
 			unlink(path);
@@ -687,47 +783,191 @@ set by the underlying system calls. See their manpages for details.
 
 =item C<ETIMEDOUT>
 
-The I<read_timeout()>, I<write_timeout()> and I<rw_timeout()> functions
+The I<read_timeout(3)>, I<write_timeout(3)> and I<rw_timeout(3)> functions
 set this when a timeout occurs.
 
 =item C<EADDRINUSE>
 
-I<fifo_open()> sets this when the path refers to a fifo that already has
+I<fifo_open(3)> sets this when the path refers to a fifo that already has
 another process reading from it.
 
 =back
 
 =head1 MT-Level
 
-MT-Safe
+I<MT-Safe>
 
-Mac OS X doesn't have I<flockfile(3)>, I<funlockfile(3)> or
-I<getc_unlocked(3)> so I<freadline()> is not MT-Safe on such platforms. You
-must guard all stdio calls in multi threaded programs with explicit
+I<Mac OS X> doesn't have I<flockfile(3)>, I<funlockfile(3)> or
+I<getc_unlocked(3)> so I<fgetline(3)> is not I<MT-Safe> on such platforms.
+You must guard all I<stdio> calls in multi-threaded programs with explicit
 synchronisation variables.
 
+=head1 EXAMPLES
+
+A paranoid I<fgetline()> example:
+
+    #include <slack/std.h>
+    #include <slack/fio.h>
+
+    int main()
+    {
+        char line[BUFSIZ];
+
+        while (fgetline(line, BUFSIZ, stdin))
+            printf("%s", line);
+
+        if (ferror(stdin))
+        {
+            if (!*line)
+                printf("%s\n", line);
+
+            return EXIT_FAILURE;
+        }
+
+        return EXIT_SUCCESS;
+    }
+
+Read from stdin but give up after 5 seconds:
+
+    #include <slack/std.h>
+    #include <slack/fio.h>
+
+    int main()
+    {
+        char buf[BUFSIZ];
+        ssize_t bytes;
+
+        if (read_timeout(STDIN_FILENO, 5, 0) == -1 ||
+            (bytes = read(STDIN_FILENO, buf, BUFSIZ)) == -1 ||
+            write(STDOUT_FILENO, buf, bytes) != bytes)
+            return EXIT_FAILURE;
+
+        return EXIT_SUCCESS;
+    }
+
+A command line sub-second sleep command:
+
+    #include <slack/std.h>
+    #include <slack/fio.h>
+
+    int main(int ac, char **av)
+    {
+        if (ac != 3)
+            return EXIT_FAILURE;
+
+        nap(atoi(av[1]), atoi(av[2]));
+
+        return EXIT_SUCCESS;
+    }
+
+Setting file flags:
+
+    #include <slack/std.h>
+    #include <slack/fio.h>
+
+    int main()
+    {
+        if (fcntl_set_flag(STDIN_FILENO, O_NONBLOCK | O_ASYNC) == -1)
+            return EXIT_FAILURE;
+
+        if (fcntl_set_flag(STDOUT_FILENO, O_APPEND) == -1)
+            return EXIT_FAILURE;
+
+        if (fcntl_set_fdflag(STDOUT_FILENO, FD_CLOEXEC) == -1)
+            return EXIT_FAILURE;
+
+        if (nonblock_on(STDOUT_FILENO) == -1)
+            return EXIT_FAILURE;
+
+        if (fcntl_clear_flag(STDIN_FILENO, O_NONBLOCK | O_ASYNC) == -1)
+            return EXIT_FAILURE;
+
+        if (fcntl_clear_flag(STDOUT_FILENO, O_APPEND) == -1)
+            return EXIT_FAILURE;
+
+        if (fcntl_clear_fdflag(STDOUT_FILENO, FD_CLOEXEC) == -1)
+            return EXIT_FAILURE;
+
+        if (nonblock_off(STDOUT_FILENO) == -1)
+            return EXIT_FAILURE;
+
+        return EXIT_SUCCESS;
+    }
+
+File locking:
+
+    #include <slack/std.h>
+    #include <slack/fio.h>
+
+    int main(int ac, char **av)
+    {
+        int fd;
+
+        if ((fd = open(av[1], O_RDWR)) == -1)
+            return EXIT_FAILURE;
+
+        if (fcntl_lock(fd, F_SETLK, F_WRLCK, SEEK_SET, 0, 0) == -1)
+            return close(fd), EXIT_FAILURE;
+
+        // Write to the file...
+
+        if (fcntl_lock(fd, F_SETLK, F_UNLCK, SEEK_SET, 0, 0) == -1)
+            return close(fd), EXIT_FAILURE;
+
+        close(fd);
+
+        return EXIT_SUCCESS;
+    }
+
+Turn a logfile into a fifo that sends log messages to syslog instead:
+
+	#include <slack/std.h>
+	#include <slack/fio.h>
+	#include <syslog.h>
+
+	int main()
+	{
+		char *fifopath = "/tmp/log2syslog";
+		char buf[BUFSIZ];
+		ssize_t bytes;
+		int fd, wfd;
+
+		if ((fd = fifo_open(fifopath, S_IRUSR | S_IWUSR | S_IWGRP | S_IWOTH, 1, &wfd)) == -1)
+			return EXIT_FAILURE;
+
+		while ((bytes = read(fd, buf, BUFSIZ)) > 0)
+		{
+			buf[bytes] = '\0';
+			syslog(LOG_DAEMON | LOG_ERR, "%s", buf);
+		}
+
+		close(fd);
+		close(wfd);
+		unlink(fifopath);
+	}
+
 =head1 BUGS
 
-Some systems, such as Mac OS X, can't lock fifos. On these systems,
-I<fifo_open()> ignores the locking failure and returns successfully. This
+Some systems, such as I<Mac OS X>, can't lock fifos. On these systems,
+I<fifo_open(3)> ignores the locking failure and returns successfully. This
 means that there is no guarantee of a unique reader process on these
 systems. You will need to lock an ordinary file yourself to provide this
 guarantee.
 
 =head1 SEE ALSO
 
-L<libslack(3)|libslack(3)>,
-L<fcntl(2)|fcntl(2)>,
-L<stat(2)|stat(2)>,
-L<fstat(2)|fstat(2)>,
-L<open(2)|open(2)>,
-L<write(2)|write(2)>,
-L<read(2)|read(2)>,
-L<mkfifo(2)|mkfifo(2)>
+I<libslack(3)>,
+I<fcntl(2)>,
+I<stat(2)>,
+I<fstat(2)>,
+I<open(2)>,
+I<write(2)>,
+I<read(2)>,
+I<mkfifo(2)>
 
 =head1 AUTHOR
 
-20011109 raf <raf@raf.org>
+20230313 raf <raf@raf.org>
 
 =cut
 
@@ -756,7 +996,7 @@ int main(int ac, char **av)
 		return EXIT_SUCCESS;
 	}
 
-	printf("Testing: fio\n");
+	printf("Testing: %s\n", "fio");
 
 	umask(0);
 
@@ -770,7 +1010,7 @@ int main(int ac, char **av)
 	else
 	{
 		if ((fcntl_lock(fd, F_SETLK, F_WRLCK, SEEK_SET, 0, 0)) != -1)
-			++errors, printf("Test2: fcntl_lock() failed\n");
+			++errors, printf("Test2: fcntl_lock(wrlock) failed\n");
 
 		/* Should really test that the following non-blocking changes do occur */
 
@@ -786,6 +1026,12 @@ int main(int ac, char **av)
 		if (fcntl_clear_flag(fd, O_NONBLOCK) == -1)
 			++errors, printf("Test6: fcntl_clear_flag() failed (%s)\n", strerror(errno));
 
+		if (fcntl_set_fdflag(fd, FD_CLOEXEC) == -1)
+			++errors, printf("Test7: fcntl_set_fdflag() failed (%s)\n", strerror(errno));
+
+		if (fcntl_clear_fdflag(fd, FD_CLOEXEC) == -1)
+			++errors, printf("Test8: fcntl_clear_fdflag() failed (%s)\n", strerror(errno));
+
 		close(fd);
 		close(wfd);
 		unlink(fifoname);
@@ -794,8 +1040,8 @@ int main(int ac, char **av)
 #define CHECK_FGETLINE(i, size, expected) \
 	if ((expected) && !fgetline(line, (size), file)) \
 		++errors, printf("Test%d: fgetline() failed\n", (i)); \
-	else if ((expected) && strcmp(line, (expected))) \
-		++errors, printf("Test%d: fgetline() read \"%s\", not \"%s\"\n", (i), line, (expected));
+	else if ((expected) && strcmp(line, ((expected) ? (expected) : ""))) \
+		++errors, printf("Test%d: fgetline() read \"%s\", not \"%s\"\n", (i), line, (expected ? expected : "(null)"));
 
 #define TEST_FGETLINE(i, buf, size, contents, line1, line2, line3) \
 	if (!(file = fopen(filename, "wb"))) \
@@ -822,40 +1068,40 @@ int main(int ac, char **av)
 		unlink(filename); \
 	}
 
-	TEST_FGETLINE(7, line, BUFSIZ, "abc\ndef\r\nghi\r", "abc\n", "def\n", "ghi\n")
-	TEST_FGETLINE(8, line, BUFSIZ, "abc\rdef\nghi\r\n", "abc\n", "def\n", "ghi\n")
-	TEST_FGETLINE(9, line, BUFSIZ, "abc\r\ndef\rghi\n", "abc\n", "def\n", "ghi\n")
-	TEST_FGETLINE(10, line, BUFSIZ, "abc\ndef\rghi", "abc\n", "def\n", "ghi")
-	TEST_FGETLINE(11, line, BUFSIZ, "", (char *)NULL, (char *)NULL, (char *)NULL)
-	TEST_FGETLINE(12, line, 5, "abc", "abc", (char *)NULL, (char *)NULL)
-	TEST_FGETLINE(13, line, 5, "abc\n", "abc\n", (char *)NULL, (char *)NULL)
-	TEST_FGETLINE(14, line, 5, "abc\r\n", "abc\n", (char *)NULL, (char *)NULL)
-	TEST_FGETLINE(15, line, 5, "abc\r", "abc\n", (char *)NULL, (char *)NULL)
-	TEST_FGETLINE(16, line, 3, "abc\r", "ab", "c\n", (char *)NULL)
-	TEST_FGETLINE(17, NULL, 0, "abc\r", (char *)NULL, (char *)NULL, (char *)NULL)
+	TEST_FGETLINE(9, line, BUFSIZ, "abc\ndef\r\nghi\r", "abc\n", "def\n", "ghi\n")
+	TEST_FGETLINE(10, line, BUFSIZ, "abc\rdef\nghi\r\n", "abc\n", "def\n", "ghi\n")
+	TEST_FGETLINE(11, line, BUFSIZ, "abc\r\ndef\rghi\n", "abc\n", "def\n", "ghi\n")
+	TEST_FGETLINE(12, line, BUFSIZ, "abc\ndef\rghi", "abc\n", "def\n", "ghi")
+	TEST_FGETLINE(13, line, BUFSIZ, "", (char *)NULL, (char *)NULL, (char *)NULL)
+	TEST_FGETLINE(14, line, 5, "abc", "abc", (char *)NULL, (char *)NULL)
+	TEST_FGETLINE(15, line, 5, "abc\n", "abc\n", (char *)NULL, (char *)NULL)
+	TEST_FGETLINE(16, line, 5, "abc\r\n", "abc\n", (char *)NULL, (char *)NULL)
+	TEST_FGETLINE(17, line, 5, "abc\r", "abc\n", (char *)NULL, (char *)NULL)
+	TEST_FGETLINE(18, line, 3, "abc\r", "ab", "c\n", (char *)NULL)
+	TEST_FGETLINE(19, NULL, 0, "abc\r", (char *)NULL, (char *)NULL, (char *)NULL)
 
 	/* Test read_timeout() and write_timeout() */
 
 	if ((fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC | O_NONBLOCK, S_IRUSR | S_IWUSR)) == -1)
-		++errors, printf("Test18: failed to create %s (%s)\n", filename, strerror(errno));
+		++errors, printf("Test20: failed to create %s (%s)\n", filename, strerror(errno));
 	else
 	{
 		char buf[12] = "0123456789\n";
 
 		if (write_timeout(fd, 1, 0) == -1)
-			++errors, printf("Test18: write_timeout(fd, 1, 0) failed (%s)\n", strerror(errno));
+			++errors, printf("Test21: write_timeout(fd, 1, 0) failed (%s)\n", strerror(errno));
 		else if (write(fd, buf, 11) != 11)
-			++errors, printf("Test18: write(fd, \"0123456789\\n\", 11) failed (%s)\n", strerror(errno));
+			++errors, printf("Test22: write(fd, \"0123456789\\n\", 11) failed (%s)\n", strerror(errno));
 		else
 		{
 			close(fd);
 
 			if ((fd = open(filename, O_RDONLY | O_NONBLOCK)) == -1)
-				++errors, printf("Test19: failed to open %s for reading (%s)\n", filename, strerror(errno));
+				++errors, printf("Test23: failed to open %s for reading (%s)\n", filename, strerror(errno));
 			else if (read_timeout(fd, 1, 0) == -1)
-				++errors, printf("Test19: read_timeout(fd, 1, 0) failed (%s)\n", strerror(errno));
+				++errors, printf("Test24: read_timeout(fd, 1, 0) failed (%s)\n", strerror(errno));
 			else if (read(fd, buf, 11) != 11)
-				++errors, printf("Test19: read(fd) failed (%s)\n", strerror(errno));
+				++errors, printf("Test25: read(fd) failed (%s)\n", strerror(errno));
 		}
 
 		close(fd);
@@ -871,26 +1117,26 @@ int main(int ac, char **av)
 	else if (errno != EINVAL) \
 		++errors, printf("Test%d: %s failed (errno = %s, not %s)\n", (i), (#func), strerror(errno), strerror(EINVAL));
 
-	TEST_ERR(20, read_timeout(-1, 0, 0))
-	TEST_ERR(21, read_timeout(0, -1, 0))
-	TEST_ERR(22, read_timeout(0, 0, -1))
-	TEST_ERR(23, write_timeout(-1, 0, 0))
-	TEST_ERR(24, write_timeout(0, -1, 0))
-	TEST_ERR(25, write_timeout(0, 0, -1))
-	TEST_ERR(26, rw_timeout(-1, 0, 0))
-	TEST_ERR(27, rw_timeout(0, -1, 0))
-	TEST_ERR(28, rw_timeout(0, 0, -1))
-	TEST_ERR(29, nap(-1, 0))
-	TEST_ERR(30, nap(0, -1))
+	TEST_ERR(26, read_timeout(-1, 0, 0))
+	TEST_ERR(27, read_timeout(0, -1, 0))
+	TEST_ERR(28, read_timeout(0, 0, -1))
+	TEST_ERR(29, write_timeout(-1, 0, 0))
+	TEST_ERR(30, write_timeout(0, -1, 0))
+	TEST_ERR(31, write_timeout(0, 0, -1))
+	TEST_ERR(32, rw_timeout(-1, 0, 0))
+	TEST_ERR(33, rw_timeout(0, -1, 0))
+	TEST_ERR(34, rw_timeout(0, 0, -1))
+	TEST_ERR(35, nap(-1, 0))
+	TEST_ERR(36, nap(0, -1))
 
 	if (errors)
-		printf("%d/30 tests failed\n", errors);
+		printf("%d/36 tests failed\n", errors);
 	else
 		printf("All tests passed\n");
 
 #ifndef HAVE_FCNTL_THAT_CAN_LOCK_FIFOS
 	printf("\n");
-	printf("    Note: Some systems (e.g. Mac OS X) can't lock fifos so fifo_open()\n");
+	printf("    Note: Some systems (e.g. FreeBSD) can't lock fifos so fifo_open()\n");
 	printf("    can't guarantee a unique reader.\n");
 #endif
 
