@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2002 raf <raf@raf.org>
+* Copyright (C) 1999-2002, 2004, 2010, 2020-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-* or visit http://www.gnu.org/copyleft/gpl.html
+* along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20020916 raf <raf@raf.org>
+* 20230313 raf <raf@raf.org>
 */
 
 /*
@@ -113,9 +111,9 @@ mappings from one object to another object, also known as hashes or
 associative arrays. I<Map>s may own their items. I<Map>s created with a
 non-C<null> destroy function use that function to destroy an item when it is
 removed from the map and to destroy each item when the map itself it
-destroyed. I<Map>s are hash tables with 11 buckets by default. They grow
+destroyed. I<Map>s are hash tables with C<11> buckets by default. They grow
 when necessary, approximately doubling in size each time up to a maximum
-size of 26,214,401 buckets.
+size of C<26,214,401> buckets.
 
 =over 4
 
@@ -123,6 +121,14 @@ size of 26,214,401 buckets.
 
 */
 
+#ifndef _BSD_SOURCE
+#define _BSD_SOURCE /* For snprintf() on OpenBSD-4.7 */
+#endif
+
+#ifndef _DEFAULT_SOURCE
+#define _DEFAULT_SOURCE /* New name for _BSD_SOURCE */
+#endif
+
 #include "config.h"
 #include "std.h"
 
@@ -274,8 +280,10 @@ static void mapping_release(Mapping *mapping)
 
 Creates a small I<Map> with string keys and C<destroy> as its item
 destructor. It is the caller's responsibility to deallocate the new map with
-I<map_release(3)> or I<map_destroy(3)>. On success, returns the new map. On
-error, returns C<null> with C<errno> set appropriately.
+I<map_release(3)> or I<map_destroy(3)>. It is strongly recommended to use
+I<map_destroy(3)>, because it also sets the pointer variable to C<null>. On
+success, returns the new map. On error, returns C<null> with C<errno> set
+appropriately.
 
 =cut
 
@@ -292,8 +300,8 @@ Map *map_create(map_release_t *destroy)
 
 Equivalent to I<map_create(3)> except that the initial number of buckets is
 approximately C<size>. The actual size will be the first prime greater than
-or equal to C<size> in a prebuilt sequence of primes between 11 and
-26,214,401 that double at each step.
+or equal to C<size> in a prebuilt sequence of primes between C<11> and
+C<26,214,401> that double at each step.
 
 =cut
 
@@ -308,9 +316,9 @@ Map *map_create_sized(size_t size, map_release_t *destroy)
 
 =item C<Map *map_create_with_hash(map_hash_t *hash, map_release_t *destroy)>
 
-Equivalent to  I<map_create(3)> except that C<hash> is used as the hash
+Equivalent to I<map_create(3)> except that C<hash> is used as the hash
 function. The arguments to C<hash> are a I<size_t> specifying the number of
-buckets and a I<const void *> specifying the key to hash. It must return a
+buckets, and a I<const void *> specifying the key to hash. It must return a
 I<size_t> between zero and the table size - 1.
 
 =cut
@@ -328,7 +336,7 @@ Map *map_create_with_hash(map_hash_t *hash, map_release_t *destroy)
 
 Equivalent to I<map_create_sized(3)> except that C<hash> is used as the hash
 function. The arguments to C<hash> are a I<size_t> specifying the number of
-buckets and a I<const void *> specifying the key to hash. It must return a
+buckets, and a I<const void *> specifying the key to hash. It must return a
 I<size_t> between zero and the table size - 1.
 
 =cut
@@ -415,7 +423,7 @@ compare mapping keys. The arguments to C<cmp> are two keys to be compared.
 It must return < 0 if the first compares less than the second, 0 if they
 compare equal and > 0 if the first compares greater than the second. C<hash>
 is the hash function. The arguments to C<hash> are a I<size_t> specifying
-the number of buckets and a I<const void *> specifying the key to hash. It
+the number of buckets, and a I<const void *> specifying the key to hash. It
 must return a I<size_t> between zero and the table size - 1. C<key_destroy>
 is the destructor for mapping keys. C<value_destroy> is the destructor for
 mapping values. On success, returns the new map. On error, returns C<null>
@@ -436,8 +444,8 @@ Map *map_create_generic(map_copy_t *copy, map_cmp_t *cmp, map_hash_t *hash, map_
 
 Equivalent to I<map_create_generic(3)> except that the initial number of
 buckets is approximately C<size>. The actual size will be the first prime
-greater than or equal to C<size> in a prebuilt sequence of primes between 11
-and 26,214,401 that double at each step.
+greater than or equal to C<size> in a prebuilt sequence of primes between C<11>
+and C<26,214,401> that double at each step.
 
 =cut
 
@@ -520,18 +528,17 @@ Map *map_create_generic_with_locker_sized(Locker *locker, size_t size, map_copy_
 =item C<int map_rdlock(const Map *map)>
 
 Claims a read lock on C<map> (if C<map> was created with a I<Locker>). This
-is needed when multiple read only L<map(3)|map(3)> module functions need to
-be called atomically. It is the client's responsibility to call
+is needed when multiple read-only I<map(3)> module functions need to be
+called atomically. It is the client's responsibility to call
 I<map_unlock(3)> after the atomic operation. The only functions that may be
 called on C<map> between calls to I<map_rdlock(3)> and I<map_unlock(3)> are
-any read only L<map(3)|map(3)> module functions whose name ends with
-C<_unlocked>. On success, returns C<0>. On error, returns an error code.
+any read-only I<map(3)> module functions whose name ends with C<_unlocked>.
+On success, returns C<0>. On error, returns an error code.
 
 =cut
 
 */
 
-
 #define map_rdlock(map) ((map) ? locker_rdlock((map)->locker) : EINVAL)
 #define map_wrlock(map) ((map) ? locker_wrlock((map)->locker) : EINVAL)
 #define map_unlock(map) ((map) ? locker_unlock((map)->locker) : EINVAL)
@@ -546,12 +553,12 @@ int (map_rdlock)(const Map *map)
 =item C<int map_wrlock(const Map *map)>
 
 Claims a write lock on C<map> (if C<map> was created with a I<Locker>). This
-is needed when multiple read/write L<map(3)|map(3)> module functions need to
-be called atomically. It is the client's responsibility to subsequently call
+is needed when multiple read/write I<map(3)> module functions need to be
+called atomically. It is the client's responsibility to subsequently call
 I<map_unlock(3)>. The only functions that may be called on C<map> between
-calls to I<map_wrlock(3)> and I<map_unlock(3)> are any L<map(3)|map(3)>
-module functions whose name ends with C<_unlocked>. On success, returns
-C<0>. On error, returns an error code.
+calls to I<map_wrlock(3)> and I<map_unlock(3)> are any I<map(3)> module
+functions whose name ends with C<_unlocked>. On success, returns C<0>. On
+error, returns an error code.
 
 =cut
 
@@ -663,7 +670,7 @@ int map_own(Map *map, map_release_t *destroy)
 
 =item C<int map_own_unlocked(Map *map, map_release_t *destroy)>
 
-Equivalent to I<map_own(3)> except that C<map> is not write locked.
+Equivalent to I<map_own(3)> except that C<map> is not write-locked.
 
 =cut
 
@@ -721,15 +728,15 @@ map_release_t *map_disown(Map *map)
 	int err;
 
 	if (!map)
-		return (map_release_t *)set_errnull(EINVAL);
+		return (map_release_t *)set_errnullf(EINVAL);
 
 	if ((err = map_wrlock(map)))
-		return (map_release_t *)set_errnull(err);
+		return (map_release_t *)set_errnullf(err);
 
 	ret = map_disown_unlocked(map);
 
 	if ((err = map_unlock(map)))
-		return (map_release_t *)set_errnull(err);
+		return (map_release_t *)set_errnullf(err);
 
 	return ret;
 }
@@ -738,7 +745,7 @@ map_release_t *map_disown(Map *map)
 
 =item C<map_release_t *map_disown_unlocked(Map *map)>
 
-Equivalent to I<map_disown(3)> except that C<map> is not write locked.
+Equivalent to I<map_disown(3)> except that C<map> is not write-locked.
 
 =cut
 
@@ -751,7 +758,7 @@ map_release_t *map_disown_unlocked(Map *map)
 	map_release_t *destroy;
 
 	if (!map)
-		return (map_release_t *)set_errnull(EINVAL);
+		return (map_release_t *)set_errnullf(EINVAL);
 
 	if (!map->value_destroy)
 		return NULL;
@@ -783,9 +790,10 @@ map_release_t *map_disown_unlocked(Map *map)
 
 C<static int map_resize(Map *map)>
 
-Resizes C<map> to use the next prime in a built in sequence of primes
-between 11 and 26,214,401 that is greater than the current size. On success,
-returns C<0>. On error, returns C<-1> with C<errno> set appropriately.
+Resizes C<map> to use the next prime in a prebuilt sequence of primes
+between C<11> and C<26,214,401> that is greater than the current size. On
+success, returns C<0>. On error, returns C<-1> with C<errno> set
+appropriately.
 
 */
 
@@ -848,6 +856,7 @@ static int map_resize(Map *map)
 	map->size = new_map->size;
 	map->items = new_map->items;
 	map->chain = new_map->chain;
+	map->value_destroy = new_map->value_destroy;
 	mem_release(new_map);
 
 	return 0;
@@ -857,9 +866,9 @@ static int map_resize(Map *map)
 
 =item C<int map_add(Map *map, const void *key, void *value)>
 
-Adds the C<(key, value)> mapping to C<map> if C<key> is not already present.
-Note that C<key> is copied but C<value> is not. On success, returns C<0>. On
-error, returns C<-1> with C<errno> set appropriately.
+Adds the C<(key, value)> mapping to C<map>, if C<key> is not already
+present. Note that C<key> is copied but C<value> is not. On success, returns
+C<0>. On error, returns C<-1> with C<errno> set appropriately.
 
 =cut
 
@@ -874,7 +883,7 @@ int map_add(Map *map, const void *key, void *value)
 
 =item C<int map_add_unlocked(Map *map, const void *key, void *value)>
 
-Equivalent to I<map_add(3)> except that C<map> is not write locked.
+Equivalent to I<map_add(3)> except that C<map> is not write-locked.
 
 =cut
 
@@ -906,7 +915,7 @@ int map_put(Map *map, const void *key, void *value)
 
 =item C<int map_put_unlocked(Map *map, const void *key, void *value)>
 
-Equivalent to I<map_put(3)> except that C<map> is not write locked.
+Equivalent to I<map_put(3)> except that C<map> is not write-locked.
 
 =cut
 
@@ -922,7 +931,7 @@ int map_put_unlocked(Map *map, const void *key, void *value)
 =item C<int map_insert(Map *map, const void *key, void *value, int replace)>
 
 Adds the C<(key, value)> mapping to C<map>, replacing any existing C<(key,
-value)> mapping if C<replace> is non-zero. Note that C<key> is copied but
+value)> mapping, if C<replace> is non-zero. Note that C<key> is copied but
 C<value> is not. On success, returns C<0>. On error, or if C<key> is already
 present and C<replace> is zero, returns C<-1> with C<errno> set
 appropriately.
@@ -954,7 +963,7 @@ int map_insert(Map *map, const void *key, void *value, int replace)
 
 =item C<int map_insert_unlocked(Map *map, const void *key, void *value, int replace)>
 
-Equivalent to I<map_insert(3)> except that C<map> is not write locked.
+Equivalent to I<map_insert(3)> except that C<map> is not write-locked.
 
 =cut
 
@@ -1048,7 +1057,7 @@ int map_remove(Map *map, const void *key)
 
 =item C<int map_remove_unlocked(Map *map, const void *key)>
 
-Equivalent to I<map_remove(3)> except that C<map> is not write locked.
+Equivalent to I<map_remove(3)> except that C<map> is not write-locked.
 
 =cut
 
@@ -1124,7 +1133,7 @@ void *map_get(Map *map, const void *key)
 
 =item C<void *map_get_unlocked(const Map *map, const void *key)>
 
-Equivalent to I<map_get(3)> except that C<map> is not read locked.
+Equivalent to I<map_get(3)> except that C<map> is not read-locked.
 
 =cut
 
@@ -1163,9 +1172,9 @@ void *map_get_unlocked(const Map *map, const void *key)
 
 =item C<Mapper *mapper_create(Map *map)>
 
-Creates an iterator for C<map>. The iterator keeps C<map> write locked until
+Creates an iterator for C<map>. The iterator keeps C<map> write-locked until
 it is released with I<mapper_release(3)> or I<mapper_destroy(3)>. Note that
-the iterator itself is not locked so it must not be shared between threads.
+the iterator itself is not locked, so it must not be shared between threads.
 On success, returns the iterator. On error, returns C<null> with C<errno>
 set appropriately.
 
@@ -1182,8 +1191,8 @@ Mapper *mapper_create(Map *map)
 
 =item C<Mapper *mapper_create_rdlocked(Map *map)>
 
-Equivalent to I<mapper_create(3)> except that C<map> is read locked rather
-than write locked. Use this in preference to I<mapper_create(3)> when no
+Equivalent to I<mapper_create(3)> except that C<map> is read-locked rather
+than write-locked. Use this in preference to I<mapper_create(3)> when no
 calls to I<mapper_remove(3)> will be made during the iteration.
 
 =cut
@@ -1208,7 +1217,7 @@ Mapper *mapper_create_rdlocked(Map *map)
 =item C<Mapper *mapper_create_wrlocked(Map *map)>
 
 Equivalent to I<mapper_create(3)> except that this function name makes the
-fact that C<map> is write locked explicit.
+fact that C<map> is write-locked explicit.
 
 =cut
 
@@ -1231,7 +1240,7 @@ Mapper *mapper_create_wrlocked(Map *map)
 
 =item C<Mapper *mapper_create_unlocked(Map *map)>
 
-Equivalent to I<mapper_create(3)> except that C<map> is not write locked.
+Equivalent to I<mapper_create(3)> except that C<map> is not write-locked.
 
 =cut
 
@@ -1433,7 +1442,7 @@ void *mapper_next(Mapper *mapper)
 
 =item C<const Mapping *mapper_next_mapping(Mapper *mapper)>
 
-Returns the next mapping (key, value pair) in the map over which C<mapper>
+Returns the next mapping (key, value) pair in the map over which C<mapper>
 is iterating. On error, returns C<null> with C<errno> set appropriately.
 
 =cut
@@ -1487,31 +1496,31 @@ void mapper_remove(Mapper *mapper)
 
 Returns whether or not there is another item in C<map> using an internal
 iterator. The first time this is called, a new internal I<Mapper> will be
-created (Note: There can be only one). When there are no more items, returns
-C<0> and destroys the internal iterator. When it returns C<1>, use
-I<map_next(3)> to retrieve the next item. On error, returns C<-1> with
-C<errno> set appropriately.
+created (Note: There can be only one at any time for a given map). When
+there are no more items, returns C<0> and destroys the internal iterator.
+When it returns C<1>, use I<map_next(3)> to retrieve the next item. On
+error, returns C<-1> with C<errno> set appropriately.
 
 Note: If an iteration using an internal iterator terminates before the end
 of the map, it is the caller's responsibility to call I<map_break(3)>.
 Failure to do so will cause the internal iterator to leak. It will also
 break the next call to I<map_has_next(3)> which will continue where the
 current iteration stopped rather than starting at the beginning again.
-I<map_release(3)> assumes that there is no internal iterator so it is the
-caller's responsibility to complete the iteration or call I<map_break(3)>
+I<map_release(3)> assumes that there is no internal iterator, so it is the
+caller's responsibility to complete the iteration, or call I<map_break(3)>
 before releasing C<map> with I<map_release(3)> or I<map_destroy(3)>.
 
 Note: The internal I<Mapper> does not lock C<map> so this function is not
 threadsafe. It can only be used with maps created in the current function
 (to guarantee that no other thread can access it). This practice should be
-observed even in single threaded applications to avoid breaking iterator
+observed even in single-threaded applications to avoid breaking iterator
 semantics (possible with nested function calls). If C<map> is a parameter or
 a variable declared outside the function, it is best to create an explicit
 I<Mapper> instead. If this function is used on such maps instead, it is the
 caller's responsibility to explicitly lock C<map> first with
 I<map_wrlock(3)> and explicitly unlock it with I<map_unlock(3)>. Do this
-even if you are writing single threaded code in case your function may one
-day be used in a multi threaded application.
+even if you are writing single-threaded code, in case your function may one
+day be used in a multi-threaded application.
 
 =cut
 
@@ -1560,7 +1569,7 @@ void map_break(Map *map)
 
 =item C<void *map_next(Map *map)>
 
-Returns the next item in C<map> using it's internal iterator. On error,
+Returns the next item in C<map> using its internal iterator. On error,
 returns C<null> with C<errno> set appropriately.
 
 =cut
@@ -1579,7 +1588,7 @@ void *map_next(Map *map)
 
 =item C<const Mapping *map_next_mapping(Map *map)>
 
-Returns the next mapping (key, value pair) in C<map> using it's internal
+Returns the next mapping (key, value) pair in C<map> using its internal
 iterator. On error, returns C<-1> with C<errno> set appropriately.
 
 =cut
@@ -1598,7 +1607,7 @@ const Mapping *map_next_mapping(Map *map)
 
 =item C<void map_remove_current(Map *map)>
 
-Removes the current item in C<map> using it's internal iterator. The next
+Removes the current item in C<map> using its internal iterator. The next
 item in the iteration is the item following the removed item, if any. This
 must be called after I<map_next(3)>.
 
@@ -1661,9 +1670,10 @@ const void *mapping_value(const Mapping *mapping)
 
 Creates and returns a list of all of the keys contained in C<map>. It is the
 caller's responsibility to deallocate the new list with I<list_release(3)>
-or I<list_destroy(3)>. The keys in the new list are owned by C<map> so the
-list returned must not outlive C<map>. On error, returns C<null> with
-C<errno> set appropriately.
+or I<list_destroy(3)>. It is strongly recommended to use I<list_destroy(3)>,
+because it also sets the pointer variable to C<null>. The keys in the new
+list are owned by C<map>, so the list returned must not outlive C<map>. On
+error, returns C<null> with C<errno> set appropriately.
 
 =cut
 
@@ -1678,7 +1688,7 @@ List *map_keys(Map *map)
 
 =item C<List *map_keys_unlocked(Map *map)>
 
-Equivalent to I<map_keys(3)> except that C<map> is not read locked.
+Equivalent to I<map_keys(3)> except that C<map> is not read-locked.
 
 =cut
 
@@ -1723,8 +1733,8 @@ List *map_keys_with_locker(Locker *locker, Map *map)
 
 =item C<List *map_keys_with_locker_unlocked(Locker *locker, Map *map)>
 
-Equivalent to I<map_keys_with_locker(3)> except that C<map> is not read
-locked.
+Equivalent to I<map_keys_with_locker(3)> except that C<map> is not
+read-locked.
 
 =cut
 
@@ -1747,7 +1757,7 @@ List *map_keys_with_locker_unlocked(Locker *locker, Map *map)
 		return NULL;
 	}
 
-	while (mapper_has_next(mapper))
+	while (mapper_has_next(mapper) == 1)
 	{
 		const Mapping *mapping = mapper_next_mapping(mapper);
 
@@ -1769,9 +1779,11 @@ List *map_keys_with_locker_unlocked(Locker *locker, Map *map)
 
 Creates and returns a list of all of the values contained in C<map>. It is
 the caller's responsibility to deallocate the new list with
-I<list_release(3)> or I<list_destroy(3)>. The values in the list are not
-owned by the list so it must not outlive the owner of the items in C<map>.
-On error, returns C<null> with C<errno> set appropriately.
+I<list_release(3)> or I<list_destroy(3)>. It is strongly recommended to use
+I<list_destroy(3)>, because it also sets the pointer variable to C<null>.
+The values in the list are not owned by the list, so it must not outlive the
+owner of the items in C<map>. On error, returns C<null> with C<errno> set
+appropriately.
 
 =cut
 
@@ -1786,7 +1798,7 @@ List *map_values(Map *map)
 
 =item C<List *map_values_unlocked(Map *map)>
 
-Equivalent to I<map_values(3)> except that C<map> is not read locked.
+Equivalent to I<map_values(3)> except that C<map> is not read-locked.
 
 =cut
 
@@ -1831,8 +1843,8 @@ List *map_values_with_locker(Locker *locker, Map *map)
 
 =item C<List *map_values_with_locker_unlocked(Locker *locker, Map *map)>
 
-Equivalent to I<map_values_with_locker(3)> except that C<map> is not read
-locked.
+Equivalent to I<map_values_with_locker(3)> except that C<map> is not
+read-locked.
 
 =cut
 
@@ -1855,7 +1867,7 @@ List *map_values_with_locker_unlocked(Locker *locker, Map *map)
 		return NULL;
 	}
 
-	while (mapper_has_next(mapper))
+	while (mapper_has_next(mapper) == 1)
 	{
 		const Mapping *mapping = mapper_next_mapping(mapper);
 
@@ -1876,7 +1888,7 @@ List *map_values_with_locker_unlocked(Locker *locker, Map *map)
 =item C<void map_apply(Map *map, map_action_t *action, void *data)>
 
 Invokes C<action> for each of C<map>'s items. The arguments passed to
-C<action> are the key, the item and C<data>. On error, sets C<errno>
+C<action> are the key, the item, and C<data>. On error, sets C<errno>
 appropriately.
 
 =cut
@@ -1892,8 +1904,8 @@ void map_apply(Map *map, map_action_t *action, void *data)
 
 =item C<void map_apply_rdlocked(Map *map, map_action_t *action, void *data)>
 
-Equivalent to I<map_apply(3)> except that C<map> is read locked rather than
-write locked. Use this in preference to I<map_apply(3)> when C<map> and its
+Equivalent to I<map_apply(3)> except that C<map> is read-locked rather than
+write-locked. Use this in preference to I<map_apply(3)> when C<map> and its
 items will not be modified during the iteration.
 
 =cut
@@ -1930,7 +1942,7 @@ void map_apply_rdlocked(Map *map, map_action_t *action, void *data)
 =item C<void map_apply_wrlocked(Map *map, map_action_t *action, void *data)>
 
 Equivalent to I<map_apply(3)> except that this function name makes the fact
-that C<map> is write locked explicit.
+that C<map> is write-locked explicit.
 
 =cut
 
@@ -1965,7 +1977,7 @@ void map_apply_wrlocked(Map *map, map_action_t *action, void *data)
 
 =item C<void map_apply_unlocked(Map *map, map_action_t *action, void *data)>
 
-Equivalent to I<map_apply(3)> except that C<map> is not write locked.
+Equivalent to I<map_apply(3)> except that C<map> is not write-locked.
 
 =cut
 
@@ -1984,7 +1996,7 @@ void map_apply_unlocked(Map *map, map_action_t *action, void *data)
 	if (!(mapper = mapper_create_unlocked(map)))
 		return;
 
-	while (mapper_has_next(mapper))
+	while (mapper_has_next(mapper) == 1)
 	{
 		const Mapping *mapping = mapper_next_mapping(mapper);
 		action(mapping->key, mapping->value, data);
@@ -2027,7 +2039,7 @@ ssize_t map_size(Map *map)
 
 =item C<ssize_t map_size_unlocked(const Map *map)>
 
-Equivalent to I<map_size(3)> except that C<map> is not read locked.
+Equivalent to I<map_size(3)> except that C<map> is not read-locked.
 
 =cut
 
@@ -2051,51 +2063,184 @@ On error, C<errno> is set either by an underlying function, or as follows:
 
 =over 4
 
-=item EINVAL
+=item C<EINVAL>
 
 When arguments are C<null> or out of range.
 
-=item ENOENT
+=item C<ENOENT>
 
-When I<map_get(3)> tries to get or I<map_remove(3)> tries to remove a
+When I<map_get(3)> tries to get, or I<map_remove(3)> tries to remove, a
 non-existent mapping.
 
 =back
 
 =head1 MT-Level
 
-MT-Disciplined
+I<MT-Disciplined>
 
-By default, I<Map>s are not MT-Safe because most programs are single
-threaded and synchronisation doesn't come for free. Even in multi threaded
-programs, not all I<Map>s are necessarily shared between multiple threads.
+By default, I<Map>s are not I<MT-Safe> because most programs are
+single-threaded, and synchronisation doesn't come for free. Even in
+multi-threaded programs, not all I<Map>s are necessarily shared between
+multiple threads.
 
 When a I<Map> is shared between multiple threads which need to be
 synchronised, the method of synchronisation must be carefully selected by
 the client code. There are tradeoffs between concurrency and overhead. The
 greater the concurrency, the greater the overhead. More locks give greater
-concurrency but have greater overhead. Readers/Writer locks can give greater
-concurrency than Mutex locks but have greater overhead. One lock for each
-I<Map> may be required, or one lock for all (or a set of) I<Map>s may be
-more appropriate.
+concurrency, but have greater overhead. Readers/Writer locks can give
+greater concurrency than Mutex, locks but have greater overhead. One lock
+for each I<Map> may be required, or one lock for all (or a set of) I<Map>s
+may be more appropriate.
 
 Generally, the best synchronisation strategy for a given application can
 only be determined by testing/benchmarking the written application. It is
 important to be able to experiment with the synchronisation strategy at this
 stage of development without pain.
 
-To facilitate this, I<Map>s can be created with I<map_create_with_locker(3)>
-which takes a I<Locker> argument. The I<Locker> specifies a lock and a set
-of functions for manipulating the lock. Each I<Map> can have it's own lock
-by creating a separate I<Locker> for each I<Map>. Multiple I<Map>s can share
-the same lock by sharing the same I<Locker>. Only the application developer
-can determine what is appropriate for each application on a case by case
-basis.
+To facilitate this, I<Map>s can be created with
+I<map_create_with_locker(3)>, which takes a I<Locker> argument. The
+I<Locker> specifies a lock and a set of functions for manipulating the lock.
+Each I<Map> can have its own lock by creating a separate I<Locker> for each
+I<Map>. Multiple I<Map>s can share the same lock by sharing the same
+I<Locker>. Only the application developer can determine what is appropriate
+for each application on a case by case basis.
 
 I<MT-Disciplined> means that the application developer has a mechanism for
 specifying the synchronisation requirements to be applied to library code.
 
-=head1 BUGS
+=head1 EXAMPLES
+
+Create a map that doesn't own its items, populate it, and then iterate over
+its values with the internal iterator to print the values:
+
+    #include <slack/std.h>
+    #include <slack/map.h>
+
+    int main()
+    {
+        Map *map;
+
+        if (!(map = map_create(NULL)))
+            return EXIT_FAILURE;
+
+        map_add(map, "abc", "123");
+        map_add(map, "def", "456");
+        map_add(map, "ghi", "789");
+
+        while (map_has_next(map) == 1)
+            printf("%s\n", (char *)map_next(map));
+
+        map_destroy(&map);
+
+        return EXIT_SUCCESS;
+    }
+
+Create a map that does own its items, populate it, and then iterate over its
+items with an external iterator to print its keys and values:
+
+    #include <slack/std.h>
+    #include <slack/map.h>
+
+    int main()
+    {
+        Map *map;
+        Mapper *mapper;
+
+        if (!(map = map_create(free)))
+            return EXIT_FAILURE;
+
+        map_add(map, "abc", strdup("123"));
+        map_add(map, "def", strdup("456"));
+        map_add(map, "ghi", strdup("789"));
+
+        if (!(mapper = mapper_create(map)))
+        {
+            map_destroy(&map);
+            return EXIT_FAILURE;
+        }
+
+        while (mapper_has_next(mapper) == 1)
+        {
+            const Mapping *mapping = mapper_next_mapping(mapper);
+
+            printf("%s -> %s\n", (char *)mapping_key(mapping), (char *)mapping_value(mapping));
+        }
+
+        mapper_destroy(&mapper);
+        map_destroy(&map);
+
+        return EXIT_SUCCESS;
+    }
+
+The same, but with an apply function:
+
+    #include <slack/std.h>
+    #include <slack/map.h>
+
+    void action(void *key, void *item, void *data)
+    {
+        printf("%s -> %s\n", (char *)key, (char *)item);
+    }
+
+    int main()
+    {
+        Map *map;
+
+        if (!(map = map_create(free)))
+            return EXIT_FAILURE;
+
+        map_add(map, "abc", strdup("123"));
+        map_add(map, "def", strdup("456"));
+        map_add(map, "ghi", strdup("789"));
+        map_apply(map, action, NULL);
+        map_destroy(&map);
+
+        return EXIT_SUCCESS;
+    }
+
+The same, but with integer keys:
+
+    #include <slack/std.h>
+    #include <slack/map.h>
+
+    static void *int_copy(const void *key)
+    {
+        return (void *)key;
+    }
+
+    static int int_cmp(const void *a, const void *b)
+    {
+        return (int)a - (int)b;
+    }
+
+    static size_t int_hash(size_t size, const void *key)
+    {
+        return (int)key % size;
+    }
+
+    void action(void *key, void *item, void *data)
+    {
+        printf("%d -> %s\n", (int)key, (char *)item);
+    }
+
+    int main(int ac, char **av)
+    {
+        Map *map;
+
+        if (!(map = map_create_generic(int_copy, int_cmp, int_hash, NULL, free)))
+            return EXIT_FAILURE;
+
+        map_add(map, (void *)123, strdup("abc"));
+        map_add(map, (void *)456, strdup("def"));
+        map_add(map, (void *)789, strdup("ghi"));
+
+        map_apply(map, action, NULL);
+        map_destroy(&map);
+
+        return EXIT_SUCCESS;
+    }
+
+=head1 CAVEAT
 
 A C<null> pointer can't be a key. Neither can zero when using integers as
 keys.
@@ -2103,19 +2248,21 @@ keys.
 If you use an internal iterator in a loop that terminates before the end of
 the map, and fail to call I<map_break(3)>, the internal iterator will leak.
 
-Uses I<malloc(3)>. The type of memory used and the allocation strategy need
-to be decoupled from this code.
+=head1 BUGS
+
+Uses I<malloc(3)> directly. The type of memory used and the allocation
+strategy should be decoupled from this code.
 
 =head1 SEE ALSO
 
-L<libslack(3)|libslack(3)>,
-L<list(3)|list(3)>,
-L<mem(3)|mem(3)>,
-L<locker(3)|locker(3)>
+I<libslack(3)>,
+I<list(3)>,
+I<mem(3)>,
+I<locker(3)>
 
 =head1 AUTHOR
 
-20020916 raf <raf@raf.org>
+20230313 raf <raf@raf.org>
 
 =cut
 
@@ -2189,7 +2336,7 @@ static void map_histogram(const char *name, Map *map)
 		size_t length = list_length(map->chain[i]);
 		if (length == -1)
 		{
-			printf("  length[%d] = -1\n", i);
+			printf("  length[%d] = -1\n", (int)i);
 			continue;
 		}
 		++histogram[length];
@@ -2199,7 +2346,7 @@ static void map_histogram(const char *name, Map *map)
 
 	for (i = 0; i < map->items; ++i)
 		if (histogram[i])
-			printf("    %d chain%s of length %d\n", histogram[i], (histogram[i] == 1) ? "" : "s", i);
+			printf("    %d chain%s of length %d\n", histogram[i], (histogram[i] == 1) ? "" : "s", (int)i);
 
 	printf("}\n");
 }
@@ -2237,7 +2384,7 @@ static void test_hash(void)
 
 	fclose(words);
 
-	printf("%d entries into %d buckets:\n\n", map->items, map->size);
+	printf("%d entries into %d buckets:\n\n", (int)map->items, (int)map->size);
 
 	for (c = 0; c < map->size; ++c)
 	{
@@ -2248,7 +2395,7 @@ static void test_hash(void)
 
 		if ((length = list_length(map->chain[c])) == -1)
 		{
-			printf(" length[%d] == -1\n", c);
+			printf(" length[%d] == -1\n", (int)c);
 			continue;
 		}
 
@@ -2260,8 +2407,8 @@ static void test_hash(void)
 	}
 
 	printf("avg = %g\n", (double)sum / (double)map->size);
-	printf("min = %d\n", min);
-	printf("max = %d\n", max);
+	printf("min = %d\n", (int)min);
+	printf("max = %d\n", (int)max);
 	map_histogram("dict", map);
 	map_release(map);
 
@@ -2345,7 +2492,11 @@ static size_t direct_hash(size_t size, int key)
 Map *mtmap = NULL;
 Locker *locker = NULL;
 pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+#ifdef PTHREAD_RWLOCK_INITIALIZER
 pthread_rwlock_t rwlock = PTHREAD_RWLOCK_INITIALIZER;
+#else
+pthread_rwlock_t rwlock;
+#endif
 int barrier[2];
 int size[2];
 const int lim = 1000;
@@ -2362,7 +2513,7 @@ void *produce(void *arg)
 		if (debug)
 			printf("p: add %d\n", i);
 
-		if (map_add(mtmap, (void *)i, (void *)i) == -1)
+		if (map_add(mtmap, (void *)(long)i, (void *)(long)i) == -1)
 			++errors, printf("Test%d: map_add(mtmap, %d), failed (%s)\n", test, i, strerror(errno));
 
 		write(size[WR], "", 1);
@@ -2386,7 +2537,7 @@ void *consume(void *arg)
 		while (read(size[RD], &ack, 1) == -1 && errno == EINTR)
 		{}
 
-		if (map_remove(mtmap, (void *)i) == -1)
+		if (map_remove(mtmap, (void *)(long)i) == -1)
 		{
 			++errors, printf("Test%d: map_remove(mtmap, %d), failed\n", test, i);
 			break;
@@ -2416,9 +2567,9 @@ void *iterate_builtin(void *arg)
 	{
 		map_wrlock(mtmap);
 
-		while (map_has_next(mtmap))
+		while (map_has_next(mtmap) == 1)
 		{
-			int val = (int)map_next(mtmap);
+			int val = (int)(long)map_next(mtmap);
 
 			if (debug)
 				printf("i%d: loop %d/%d val %d\n", t, i, lim / 10, val);
@@ -2452,7 +2603,7 @@ void *iterate_rdlocked(void *arg)
 
 		while (mapper_has_next(mapper) == 1)
 		{
-			int val = (int)mapper_next(mapper);
+			int val = (int)(long)mapper_next(mapper);
 
 			if (debug)
 				printf("j%d: loop %d/%d val %d\n", t, i, lim / 10, val);
@@ -2479,7 +2630,7 @@ void *iterate_wrlocked(void *arg)
 
 		while (mapper_has_next(mapper) == 1)
 		{
-			int val = (int)mapper_next(mapper);
+			int val = (int)(long)mapper_next(mapper);
 
 			if (debug)
 				printf("k%d: loop %d/%d val %d\n", t, i, lim / 10, val);
@@ -2513,14 +2664,14 @@ void *reader(void *arg)
 
 		map_rdlock(mtmap);
 		key = 1 + (int)((double)(map_size_unlocked(mtmap) - 1) * r / (RAND_MAX + 1.0));
-		value = (int)map_get_unlocked(mtmap, (void *)key);
+		value = (int)(long)map_get_unlocked(mtmap, (void *)(long)key);
 		map_unlock(mtmap);
 
 		keys = map_keys(mtmap);
 		values = map_values(mtmap);
 
 		if (debug)
-			printf("r%d: loop %d/%d key/val %d/%d, #keys %d, #values %d\n", t, i, lim / 10, key, value, list_length(keys), list_length(values));
+			printf("r%d: loop %d/%d key/val %d/%d, #keys %d, #values %d\n", t, i, lim / 10, key, value, (int)list_length(keys), (int)list_length(values));
 
 		list_destroy(&keys);
 		list_destroy(&values);
@@ -2610,7 +2761,7 @@ void mt_test(int test, Locker *locker)
 
 #define CHECK_LIST(i, desc, list) \
 	if (list_length(list) != 4) \
-		++errors, printf("Test%d: %s failed (%d items, not 4)\n", (i), (desc), list_length(list)); \
+		++errors, printf("Test%d: %s failed (%d items, not 4)\n", (i), (desc), (int)list_length(list)); \
 	else \
 	{ \
 		TEST_ACT((i), list_sort((list), (list_cmp_t *)sort_cmp)); \
@@ -2719,7 +2870,7 @@ int main(int ac, char **av)
 
 		TEST_ACT(21, keys = list_create(NULL))
 
-		while (map_has_next(map))
+		while (map_has_next(map) == 1)
 		{
 			void *item;
 
@@ -2735,10 +2886,10 @@ int main(int ac, char **av)
 		TEST_ACT(24, keys = list_create(NULL))
 		TEST_ACT(24, values = list_create(NULL))
 
-		while (map_has_next(map))
+		while (map_has_next(map) == 1)
 		{
 			const Mapping *mapping;
-			
+
 			TEST_ACT(25, mapping = map_next_mapping(map))
 			else
 			{
@@ -2835,7 +2986,7 @@ int main(int ac, char **av)
 			while (mapper_has_next(mapper) == 1)
 			{
 				void *item;
-				
+
 				TEST_ACT(64, item = mapper_next(mapper))
 				mapper_remove(mapper);
 			}
@@ -2862,10 +3013,10 @@ int main(int ac, char **av)
 		TEST_ACT(71, map_add(map, "3", "3") == 0)
 		TEST_ACT(72, map_add(map, "4", "4") == 0)
 
-		while (map_has_next(map))
+		while (map_has_next(map) == 1)
 		{
 			void *item;
-			
+
 			TEST_ACT(73, item = map_next(map))
 			map_remove_current(map);
 		}
@@ -2961,31 +3112,31 @@ int main(int ac, char **av)
 		TEST_ACT(130, map_add(map, (void *)25, (void *)25) != 0)
 		TEST_EQ(130, map_size(map), 25)
 
-		TEST_EQ(131, (int)map_get(map, (void *)1), 1)
-		TEST_EQ(132, (int)map_get(map, (void *)2), 2)
-		TEST_EQ(133, (int)map_get(map, (void *)3), 3)
-		TEST_EQ(134, (int)map_get(map, (void *)4), 4)
-		TEST_EQ(135, (int)map_get(map, (void *)5), 5)
-		TEST_EQ(136, (int)map_get(map, (void *)6), 6)
-		TEST_EQ(137, (int)map_get(map, (void *)7), 7)
-		TEST_EQ(138, (int)map_get(map, (void *)8), 8)
-		TEST_EQ(139, (int)map_get(map, (void *)9), 9)
-		TEST_EQ(140, (int)map_get(map, (void *)10), 10)
-		TEST_EQ(141, (int)map_get(map, (void *)11), 11)
-		TEST_EQ(142, (int)map_get(map, (void *)12), 12)
-		TEST_EQ(143, (int)map_get(map, (void *)13), 13)
-		TEST_EQ(144, (int)map_get(map, (void *)14), 14)
-		TEST_EQ(145, (int)map_get(map, (void *)15), 15)
-		TEST_EQ(146, (int)map_get(map, (void *)16), 16)
-		TEST_EQ(147, (int)map_get(map, (void *)17), 17)
-		TEST_EQ(148, (int)map_get(map, (void *)18), 18)
-		TEST_EQ(149, (int)map_get(map, (void *)19), 19)
-		TEST_EQ(150, (int)map_get(map, (void *)20), 20)
-		TEST_EQ(151, (int)map_get(map, (void *)21), 21)
-		TEST_EQ(152, (int)map_get(map, (void *)22), 22)
-		TEST_EQ(153, (int)map_get(map, (void *)23), 23)
-		TEST_EQ(154, (int)map_get(map, (void *)24), 24)
-		TEST_EQ(155, (int)map_get(map, (void *)25), 25)
+		TEST_EQ(131, (int)(long)map_get(map, (void *)1), 1)
+		TEST_EQ(132, (int)(long)map_get(map, (void *)2), 2)
+		TEST_EQ(133, (int)(long)map_get(map, (void *)3), 3)
+		TEST_EQ(134, (int)(long)map_get(map, (void *)4), 4)
+		TEST_EQ(135, (int)(long)map_get(map, (void *)5), 5)
+		TEST_EQ(136, (int)(long)map_get(map, (void *)6), 6)
+		TEST_EQ(137, (int)(long)map_get(map, (void *)7), 7)
+		TEST_EQ(138, (int)(long)map_get(map, (void *)8), 8)
+		TEST_EQ(139, (int)(long)map_get(map, (void *)9), 9)
+		TEST_EQ(140, (int)(long)map_get(map, (void *)10), 10)
+		TEST_EQ(141, (int)(long)map_get(map, (void *)11), 11)
+		TEST_EQ(142, (int)(long)map_get(map, (void *)12), 12)
+		TEST_EQ(143, (int)(long)map_get(map, (void *)13), 13)
+		TEST_EQ(144, (int)(long)map_get(map, (void *)14), 14)
+		TEST_EQ(145, (int)(long)map_get(map, (void *)15), 15)
+		TEST_EQ(146, (int)(long)map_get(map, (void *)16), 16)
+		TEST_EQ(147, (int)(long)map_get(map, (void *)17), 17)
+		TEST_EQ(148, (int)(long)map_get(map, (void *)18), 18)
+		TEST_EQ(149, (int)(long)map_get(map, (void *)19), 19)
+		TEST_EQ(150, (int)(long)map_get(map, (void *)20), 20)
+		TEST_EQ(151, (int)(long)map_get(map, (void *)21), 21)
+		TEST_EQ(152, (int)(long)map_get(map, (void *)22), 22)
+		TEST_EQ(153, (int)(long)map_get(map, (void *)23), 23)
+		TEST_EQ(154, (int)(long)map_get(map, (void *)24), 24)
+		TEST_EQ(155, (int)(long)map_get(map, (void *)25), 25)
 		map_destroy(&map);
 	}
 
@@ -2996,6 +3147,10 @@ int main(int ac, char **av)
 	if (debug)
 		setbuf(stdout, NULL);
 
+#ifndef PTHREAD_RWLOCK_INITIALIZER
+	pthread_rwlock_init(&rwlock, NULL);
+#endif
+
 	if (debug)
 		locker = locker_create_debug_rwlock(&rwlock);
 	else
@@ -3025,7 +3180,7 @@ int main(int ac, char **av)
 	/* Test assumption: sizeof(int) <= sizeof(void *) */
 
 	if (sizeof(int) > sizeof(void *))
-		++errors, printf("Test 222: assumption failed: sizeof(int) > sizeof(void *): int maps are limited to %d bytes\n", sizeof(void *));
+		++errors, printf("Test 222: assumption failed: sizeof(int) > sizeof(void *): int maps are limited to %d bytes\n", (int)sizeof(void *));
 
 	/* Test assumption: memset(&ptr, 0, sizeof(void *)) same as NULL */
 
