@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2002 raf <raf@raf.org>
+* Copyright (C) 1999-2002, 2004, 2010, 2020-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-* or visit http://www.gnu.org/copyleft/gpl.html
+* along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20020916 raf <raf@raf.org>
+* 20230313 raf <raf@raf.org>
 */
 
 /*
@@ -34,6 +32,7 @@ I<libslack(msg)> - message module
 
     typedef struct Msg Msg;
     typedef void msg_out_t(void *data, const void *mesg, size_t mesglen);
+    typedef int msg_filter_t(void **mesgp, const void *mesg, size_t mesglen);
     typedef void msg_release_t(void *data);
 
     Msg *msg_create(int type, msg_out_t *out, void *data, msg_release_t *destroy);
@@ -65,6 +64,8 @@ I<libslack(msg)> - message module
     Msg *msg_create_plex_with_locker(Locker *locker, Msg *msg1, Msg *msg2);
     int msg_add_plex(Msg *mesg, Msg *item);
     int msg_add_plex_unlocked(Msg *mesg, Msg *item);
+    Msg *msg_create_filter(msg_filter_t *filter, Msg *mesg);
+    Msg *msg_create_filter_with_locker(Locker *locker, msg_filter_t *filter, Msg *mesg);
     const char *msg_set_timestamp_format(const char *format);
     int msg_set_timestamp_format_locker(Locker *locker);
     int syslog_lookup_facility(const char *facility);
@@ -91,6 +92,14 @@ priority names and codes.
 
 */
 
+#ifndef _BSD_SOURCE
+#define _BSD_SOURCE /* For snprintf() on OpenBSD-4.7 */
+#endif
+
+#ifndef _DEFAULT_SOURCE
+#define _DEFAULT_SOURCE /* New name for _BSD_SOURCE */
+#endif
+
 #include "config.h"
 #include "std.h"
 
@@ -112,12 +121,14 @@ priority names and codes.
 typedef int MsgFDData;
 typedef struct MsgFileData MsgFileData;
 typedef struct MsgSyslogData MsgSyslogData;
+typedef struct MsgFilterData MsgFilterData;
 typedef struct MsgPlexData MsgPlexData;
 
 #define MSG_FD 1
 #define MSG_FILE 2
 #define MSG_SYSLOG 3
 #define MSG_PLEX 4
+#define MSG_FILTER 5
 
 struct Msg
 {
@@ -146,6 +157,12 @@ struct MsgPlexData
 	Msg **list;    /* list of Msg objects */
 };
 
+struct MsgFilterData
+{
+	msg_filter_t *filter; /* filter function */
+	Msg *mesg;            /* destination Msg */
+};
+
 typedef struct syslog_map_t syslog_map_t;
 
 struct syslog_map_t
@@ -155,7 +172,7 @@ struct syslog_map_t
 };
 
 /*
-** The following masks may be wrong on some systems.
+** The following masks might be wrong on some systems.
 */
 
 #ifndef LOG_PRIMASK
@@ -214,10 +231,11 @@ static Locker *timestamp_format_locker = NULL;
 =item C<Msg *msg_create(int type, msg_out_t *out, void *data, msg_release_t *destroy)>
 
 Creates a I<Msg> object initialised with C<type>, C<out>, C<data> and
-C<destroy>. Client defined message handlers must specify a C<type> greater
-than C<4>. It is the caller's responsibility to deallocate the new I<Msg>
-with I<msg_release(3)> or I<msg_destroy>. On success, returns the new I<Msg>
-object. On error, returns C<null>.
+C<destroy>. Client-defined message handlers must specify a C<type> greater
+than C<5>. It is the caller's responsibility to deallocate the new I<Msg>
+with I<msg_release(3)> or I<msg_destroy>. It is strongly recommended to use
+I<msg_destroy(3)>, because it also sets the pointer variable to C<null>. On
+success, returns the new I<Msg> object. On error, returns C<null>.
 
 =cut
 
@@ -260,11 +278,11 @@ Msg *msg_create_with_locker(Locker *locker, int type, msg_out_t *out, void *data
 =item C<int msg_rdlock(Msg *mesg)>
 
 Claims a read lock on C<mesg> (if C<mesg> was created with a I<Locker>).
-This is needed when multiple read only L<msg(3)|msg(3)> module functions
-need to be called atomically. It is the caller's responsbility to call
+This is needed when multiple read-only I<msg(3)> module functions
+need to be called atomically. It is the caller's responsibility to call
 I<msg_unlock(3)> after the atomic operation. The only functions that may be
 called on C<mesg> between calls to I<msg_rdlock(3)> and I<msg_unlock(3)> are
-any read only L<msg(3)|msg(3)> module functions whose name ends with
+any read-only I<msg(3)> module functions whose name ends with
 C<_unlocked>. On success, returns C<0>. On error, returns an error code.
 
 =cut
@@ -287,11 +305,11 @@ int (msg_rdlock)(Msg *mesg)
 Claims a write lock on C<mesg>.
 
 Claims a write lock on C<mesg> (if C<mesg> was created with a I<Locker>).
-This is needed when multiple read/write L<msg(3)|msg(3)> module functions
-need to be called atomically. It is the caller's responsbility to call
+This is needed when multiple read/write I<msg(3)> module functions
+need to be called atomically. It is the caller's responsibility to call
 I<msg_unlock(3)> after the atomic operation. The only functions that may be
 called on C<mesg> between calls to I<msg_rdlock(3)> and I<msg_unlock(3)> are
-any L<msg(3)|msg(3)> module functions whose name ends with C<_unlocked>. On
+any I<msg(3)> module functions whose name ends with C<_unlocked>. On
 success, returns C<0>. On error, returns an error code.
 
 =cut
@@ -372,8 +390,8 @@ Any remaining arguments are processed as in I<printf(3)>.
 B<Warning: Do not under any circumstances ever pass a non-literal string as
 the format argument unless you know exactly how many conversions will take
 place. Being careless with this is a very good way to build potential
-security holes into your programs. The same is true for all functions that
-take a printf()-like format string as an argument.>
+security vulnerabilities into your programs. The same is true for all
+functions that take a printf()-like format string as an argument.>
 
     msg_out(dst, buf);       // EVIL
     msg_out(dst, "%s", buf); // GOOD
@@ -394,7 +412,7 @@ void msg_out(Msg *dst, const char *format, ...)
 
 =item C<void msg_out_unlocked(Msg *dst, const char *format, ...)>
 
-Equivalent to I<msg_out(3)> except that C<dst> is not read locked.
+Equivalent to I<msg_out(3)> except that C<dst> is not read-locked.
 
 =cut
 
@@ -442,7 +460,7 @@ void vmsg_out(Msg *dst, const char *format, va_list args)
 
 =item C<void vmsg_out_unlocked(Msg *dst, const char *format, va_list args)>
 
-Equivalent to I<vmsg_out(3)> except that C<dst> is not read locked.
+Equivalent to I<vmsg_out(3)> except that C<dst> is not read-locked.
 
 =cut
 
@@ -502,14 +520,15 @@ static void msg_fddata_release(MsgFDData *data)
 C<void msg_out_fd(void *data, const void *mesg, size_t mesglen)>
 
 Sends a message to a file descriptor. C<data> is a pointer to the file
-descriptor. C<mesg> is the message. C<mesglen> is it's length.
+descriptor. C<mesg> is the message. C<mesglen> is its length.
 
 */
 
 static void msg_out_fd(void *data, const void *mesg, size_t mesglen)
 {
 	if (data && mesg)
-		write(*(MsgFDData *)data, mesg, mesglen);
+		if (write(*(MsgFDData *)data, mesg, mesglen) == -1)
+			/* Avoid gcc warning */;
 }
 
 /*
@@ -564,8 +583,9 @@ Msg *msg_create_fd_with_locker(Locker *locker, int fd)
 
 Creates a I<Msg> object that sends messages to standard error. It is the
 caller's responsibility to deallocate the new I<Msg> with I<msg_release(3)>
-or I<msg_destroy(3)>. On success, returns the new I<Msg> object. On error,
-returns C<null>.
+or I<msg_destroy(3)>. It is strongly recommended to use I<msg_destroy(3)>,
+because it also sets the pointer variable to C<null>. On success, returns
+the new I<Msg> object. On error, returns C<null>.
 
 =cut
 
@@ -598,8 +618,9 @@ Msg *msg_create_stderr_with_locker(Locker *locker)
 
 Creates a I<Msg> object that sends messages to standard output. It is the
 caller's responsibility to deallocate the new I<Msg> with I<msg_release(3)>
-or I<msg_destroy(3)>. On success, returns the new I<Msg> object. On error,
-returns C<null>.
+or I<msg_destroy(3)>. It is strongly recommended to use I<msg_destroy(3)>,
+because it also sets the pointer variable to C<null>. On success, returns
+the new I<Msg> object. On error, returns C<null>.
 
 =cut
 
@@ -633,8 +654,8 @@ C<int msg_filedata_init(MsgFileData *data, const char *path)>
 Initialises the internal data needed by a I<Msg> object that sends messages
 to the file specified by C<path>. This data consists of a copy of C<path>
 and an open file descriptor to the file. The file descriptor is opened with
-the C<O_WRONLY>, C<O_CREAT> and C<O_APPEND> flags. On success, returns C<0>.
-On error, returns C<-1> with C<errno> set appropriately.
+the C<O_WRONLY>, C<O_CREAT>, and C<O_APPEND> flags. On success, returns
+C<0>. On error, returns C<-1> with C<errno> set appropriately.
 
 */
 
@@ -704,7 +725,7 @@ static void msg_filedata_release(MsgFileData *data)
 C<void msg_out_file(void *data, const void *mesg, size_t mesglen)>
 
 Sends a message to a file. C<data> contains the file descriptor. C<mesg> is
-the message. C<mesglen> is it's length. On error, sets C<errno>
+the message. C<mesglen> is its length. On error, sets C<errno>
 appropriately.
 
 */
@@ -738,7 +759,8 @@ static void msg_out_file(void *data, const void *mesg, size_t mesglen)
 	memmove(buf + buflen, mesg, mesglen);
 
 	if (mesg && dst && dst->fd != -1)
-		write(dst->fd, buf, buflen + mesglen);
+		if (write(dst->fd, buf, buflen + mesglen) == -1)
+			/* Avoid gcc warning */;
 }
 
 /*
@@ -747,8 +769,10 @@ static void msg_out_file(void *data, const void *mesg, size_t mesglen)
 
 Creates a I<Msg> object that sends messages to the file specified by
 C<path>. It is the caller's responsibility to deallocate the new I<Msg> with
-I<msg_release(3)> or I<msg_destroy(3)>. On success, returns the new I<Msg>
-object. On error, returns C<null> with C<errno> set appropriately.
+I<msg_release(3)> or I<msg_destroy(3)>. It is strongly recommended to use
+I<msg_destroy(3)>, because it also sets the pointer variable to C<null>. On
+success, returns the new I<Msg> object. On error, returns C<null> with
+C<errno> set appropriately.
 
 =cut
 
@@ -861,8 +885,9 @@ static void msg_sysdata_release(MsgSyslogData *data)
 
 C<void msg_out_syslog(void *data, const void *mesg, size_t mesglen)>
 
-Sends a message to I<syslog>. C<data> contains the facility to use. C<mesg>
-is the message. C<mesglen> is it's length.
+Sends a message to I<syslog>. C<data> is a pointer to a C<MsgSyslogData>
+object that contains the facility and priority to use. C<mesg> is the
+message. C<mesglen> is its length.
 
 */
 
@@ -881,8 +906,10 @@ static void msg_out_syslog(void *data, const void *mesg, size_t mesglen)
 Creates a I<Msg> object that sends messages to I<syslog> initialised with
 C<ident>, C<option>, C<facility> and C<priority>. It is the caller's
 responsibility to deallocate the new I<Msg> with I<msg_release(3)> or
-I<msg_destroy(3)>. On success, returns the new I<Msg> object. On error,
-returns C<null> with C<errno> set appropriately.
+I<msg_destroy(3)>. It is strongly recommended to use I<msg_destroy(3)>,
+because it also sets the pointer variable to C<null>. On success, returns
+the new I<Msg> object. On error, returns C<null> with C<errno> set
+appropriately.
 
 =cut
 
@@ -955,8 +982,8 @@ Msg *msg_syslog_set_facility(Msg *mesg, int facility)
 
 =item C<Msg *msg_syslog_set_facility_unlocked(Msg *mesg, int facility)>
 
-Equivalent to I<msg_syslog_set_facility(3)> except that C<mesg> is not write
-locked.
+Equivalent to I<msg_syslog_set_facility(3)> except that C<mesg> is not
+write-locked.
 
 =cut
 
@@ -1009,8 +1036,8 @@ Msg *msg_syslog_set_priority(Msg *mesg, int priority)
 
 =item C<Msg *msg_syslog_set_priority_unlocked(Msg *mesg, int priority)>
 
-Equivalent to I<msg_syslog_set_priority(3)> except that C<mesg> is not write
-locked.
+Equivalent to I<msg_syslog_set_priority(3)> except that C<mesg> is not
+write-locked.
 
 =cut
 
@@ -1134,7 +1161,7 @@ static void msg_plexdata_release(MsgPlexData *data)
 C<void msg_out_plex(void *data, const void *mesg, size_t mesglen)>
 
 Multiplexes a message to several I<Msg> objects. C<data> contains the list
-of I<Msg> objects. C<mesg> is the message. C<mesglen> is it's length.
+of I<Msg> objects. C<mesg> is the message. C<mesglen> is its length.
 
 */
 
@@ -1161,8 +1188,10 @@ static void msg_out_plex(void *data, const void *mesg, size_t mesglen)
 Creates a I<Msg> object that multiplexes messages to C<msg1> and C<msg2>.
 Further I<Msg> objects may be added to its list using I<msg_add_plex(3)>. It
 is the caller's responsibility to deallocate the new I<Msg> with
-I<msg_release(3)> or I<msg_destroy(3)>. On success, returns the new I<Msg>
-object. On error, returns C<null> with C<errno> set appropriately.
+I<msg_release(3)> or I<msg_destroy(3)>. It is strongly recommended to use
+I<msg_destroy(3)>, because it also sets the pointer variable to C<null>. On
+success, returns the new I<Msg> object. On error, returns C<null> with
+C<errno> set appropriately.
 
 =cut
 
@@ -1236,7 +1265,7 @@ int msg_add_plex(Msg *mesg, Msg *item)
 
 =item C<int msg_add_plex_unlocked(Msg *mesg, Msg *item)>
 
-Equivalent to I<msg_add_plex(3)> except that C<mesg> is not write locked.
+Equivalent to I<msg_add_plex(3)> except that C<mesg> is not write-locked.
 
 =cut
 
@@ -1252,6 +1281,127 @@ int msg_add_plex_unlocked(Msg *mesg, Msg *item)
 
 /*
 
+C<MsgFilterData *msg_filterdata_create(msg_filter_t *filter, Msg *mesg)>
+
+Creates the internal data needed by a I<Msg> object that sends filtered
+messages to another I<Msg> object, I<mesg>. C<filter> and C<mesg> are used
+to initialise a I<msg> object. On success, returns the data. On error,
+returns C<null> with C<errno> set appropriately.
+
+*/
+
+static MsgFilterData *msg_filterdata_create(msg_filter_t *filter, Msg *mesg)
+{
+	MsgFilterData *data;
+
+	if (!filter || !mesg)
+		return set_errnull(EINVAL);
+
+	if (!(data = mem_new(MsgFilterData)))
+		return NULL;
+
+	data->filter = filter;
+	data->mesg = mesg;
+
+	return data;
+}
+
+/*
+
+C<void msg_filterdata_release(MsgFilterData *data)>
+
+Releases (deallocates) the internal data needed by a I<Msg> object that
+sends filtered messages to another I<Msg> object.
+
+*/
+
+static void msg_filterdata_release(MsgFilterData *data)
+{
+	if (!data)
+		return;
+
+	msg_release(data->mesg);
+	mem_release(data);
+}
+
+/*
+
+C<void msg_out_filter(void *data, const void *mesg, size_t mesglen)>
+
+Filters and sends a message to another I<Msg> object. C<data> contains the
+filter function and the destination I<Msg> object. C<mesg> is the unfiltered
+message. C<mesglen> is its length.
+
+*/
+
+static void msg_out_filter(void *data, const void *mesg, size_t mesglen)
+{
+	MsgFilterData *filter_data = data;
+	void *filtered_mesg;
+	int filtered_mesglen;
+
+	if (!filter_data || !mesg || !mesglen)
+		return;
+
+	if ((filtered_mesglen = filter_data->filter(&filtered_mesg, mesg, mesglen)) == -1)
+		return;
+
+	filter_data->mesg->out(filter_data->mesg->data, filtered_mesg, filtered_mesglen);
+	mem_release(filtered_mesg);
+}
+
+/*
+
+=item C<Msg *msg_create_filter(msg_filter_t *filter, Msg *mesg)>
+
+Creates a I<Msg> object that sends messages to I<mesg> after filtering
+messages through the I<filter> function which must dynamically create a
+modified version of its input message which will be deallocated by its
+caller. It is the caller's responsibility to deallocate the new I<Msg> with
+I<msg_release(3)> or I<msg_destroy(3)>. It is strongly recommended to use
+I<msg_destroy(3)>, because it also sets the pointer variable to C<null>. On
+success, returns the new I<Msg> object. On error, returns C<null> with
+C<errno> set appropriately.
+
+=cut
+
+*/
+
+Msg *msg_create_filter(msg_filter_t *filter, Msg *mesg)
+{
+	return msg_create_filter_with_locker(NULL, filter, mesg);
+}
+
+/*
+
+=item C<Msg *msg_create_filter_with_locker(Locker *locker, msg_filter_t *filter, Msg *mesg)>
+
+Equivalent to I<msg_create_filter(3)> except that multiple threads accessing
+the new I<Msg> will be synchronised by C<locker>.
+
+=cut
+
+*/
+
+Msg *msg_create_filter_with_locker(Locker *locker, msg_filter_t *filter, Msg *mesg)
+{
+	MsgFilterData *data;
+	Msg *newmesg;
+
+	if (!(data = msg_filterdata_create(filter, mesg)))
+		return NULL;
+
+	if (!(newmesg = msg_create_with_locker(locker, MSG_FILTER, msg_out_filter, data, (msg_release_t *)msg_filterdata_release)))
+	{
+		msg_filterdata_release(data);
+		return NULL;
+	}
+
+	return newmesg;
+}
+
+/*
+
 =item C<const char *msg_set_timestamp_format(const char *format)>
 
 Sets the I<strftime(3)> format string used when sending messages to a file.
@@ -1417,6 +1567,9 @@ C<facility>. If C<priority> is non-C<null> the parsed priority is stored in
 the location pointed to by C<priority>. On success, returns C<0>. On error,
 returns C<-1> with C<errno> set appropriately.
 
+These are the supported syslog facility and priority names and their
+corresponding symbolic constants:
+
     syslog facilities          syslog priorities
     ----------------------     -----------------------
     "kern"      LOG_KERN       "emerg"       LOG_EMERG
@@ -1472,7 +1625,6 @@ int syslog_parse(const char *spec, int *facility, int *priority)
 	return 0;
 }
 
-
 /*
 
 =back
@@ -1483,15 +1635,22 @@ On error, C<errno> is set by underlying functions or as follows:
 
 =over 4
 
-=item EINVAL
+=item C<EINVAL>
 
-An argument was C<null> or could not be parsed.
+An argument was C<null>, or could not be parsed.
 
 =back
 
+=head1 MT-Level
+
+I<MT-Disciplined> - msg functions - See I<locker(3)> for details.
+
+I<MT-Safe> - syslog functions
+
 =head1 EXAMPLE
 
-    #include <syslog.h>
+Parse syslog facility priority pair:
+
     #include <slack/std.h>
     #include <slack/msg.h>
 
@@ -1505,24 +1664,40 @@ An argument was C<null> or could not be parsed.
         return EXIT_SUCCESS;
     }
 
-=head1 MT-Level
+Multiplex a message to several locations:
 
-MT-Disciplined - msg functions - See L<locker(3)|locker(3)> for details
+    #include <slack/std.h>
+    #include <slack/msg.h>
 
-MT-Safe - syslog functions
+    int main(int ac, char **av)
+    {
+        Msg *stdout_msg = msg_create_stdout();
+        Msg *stderr_msg = msg_create_stderr();
+        Msg *file_msg = msg_create_file("/tmp/junk");
+        Msg *syslog_msg = msg_create_syslog("ident", 0, LOG_DAEMON, LOG_ERR);
+        Msg *plex_msg = msg_create_plex(stdout_msg, stderr_msg);
+
+        msg_add_plex(plex_msg, file_msg);
+        msg_add_plex(plex_msg, syslog_msg);
+
+        msg_out(plex_msg, "Multiplex message\n");
+        unlink("/tmp/junk");
+
+        return EXIT_SUCCESS;
+    }
 
 =head1 SEE ALSO
 
-L<libslack(3)|libslack(3)>,
-L<err(3)|err(3)>,
-L<prog(3)|prog(3)>,
-L<openlog(3)|openlog(3)>,
-L<syslog(3)|syslog(3)>,
-L<locker(3)|locker(3)>
+I<libslack(3)>,
+I<err(3)>,
+I<prog(3)>,
+I<openlog(3)>,
+I<syslog(3)>,
+I<locker(3)>
 
 =head1 AUTHOR
 
-20020916 raf <raf@raf.org>
+20230313 raf <raf@raf.org>
 
 =cut
 
@@ -1532,7 +1707,7 @@ L<locker(3)|locker(3)>
 
 #ifdef TEST
 
-static int verify(const char *prefix, const char *name, const char *mesg)
+static int verify(int test, const char *name, const char *mesg)
 {
 	char buf[MSG_SIZE];
 	int fd;
@@ -1540,7 +1715,7 @@ static int verify(const char *prefix, const char *name, const char *mesg)
 
 	if ((fd = open(name, O_RDONLY)) == -1)
 	{
-		printf("%s: failed to create msg file: %s (%s)\n", prefix, name, strerror(errno));
+		printf("Test%d: failed to create msg file: %s (%s)\n", test, name, strerror(errno));
 		return 1;
 	}
 
@@ -1551,31 +1726,43 @@ static int verify(const char *prefix, const char *name, const char *mesg)
 
 	if (bytes == -1)
 	{
-		printf("%s: failed to read msg file: %s (%s)\n", prefix, name, strerror(errno));
+		printf("Test%d: failed to read msg file: %s (%s)\n", test, name, strerror(errno));
 		return 1;
 	}
 
 	if (!strstr(buf, mesg))
 	{
-		printf("%s: msg file produced incorrect input:\nshould contain:\n%s\nwas:\n%s\n", prefix, mesg, buf);
+		printf("Test%d: msg file produced incorrect input:\nshould contain:\n%s\nwas:\n%s\n", test, mesg, buf);
 		return 1;
 	}
 
 	return 0;
 }
 
+static int prefix_filter(void **mesgp, const void *mesg, size_t mesglen)
+{
+	if (!mesgp || !mesg)
+		return -1;
+
+	return asprintf((char **)mesgp, "[%d] %.*s", 12345, (int)mesglen, (char *)mesg);
+}
+
 int main(int ac, char **av)
 {
 	const char *msg_file_name = "./msg.file";
 	const char *msg_stdout_name = "./msg.stdout";
 	const char *msg_stderr_name = "./msg.stderr";
-	const char *mesg = "multiplexed msg to stdout, stderr, ./msg.file and syslog local0.debug\n";
+	const char *msg_filter_name = "./msg.filter";
+	const char *mesg = "multiplexed msg to stdout, stderr, ./msg.file, syslog local0.debug and ./msg.filtered\n";
 	const char *note = "\n    Note: Can't verify syslog local0.debug. Look for:";
+	void *filtered_mesg = null;
+	int filtered_mesglen = 0;
 
 	Msg *msg_stdout = msg_create_stdout();
 	Msg *msg_stderr = msg_create_stderr();
 	Msg *msg_file = msg_create_file(msg_file_name);
 	Msg *msg_syslog = msg_create_syslog(NULL, 0, LOG_LOCAL0, LOG_DEBUG);
+	Msg *msg_filter = msg_create_filter(prefix_filter, msg_create_file(msg_filter_name));
 	Msg *msg_plex = msg_create_plex(msg_stdout, msg_stderr);
 	int errors = 0;
 	int tests = 0;
@@ -1589,20 +1776,33 @@ int main(int ac, char **av)
 
 	printf("Testing: %s\n", "msg");
 
+	++tests;
 	if (!msg_stdout)
-		++errors, printf("Test1: failed to create msg_stdout");
+		++errors, printf("Test%d: failed to create msg_stdout\n", tests);
+	++tests;
 	if (!msg_stderr)
-		++errors, printf("Test2: failed to create msg_stder");
+		++errors, printf("Test%d: failed to create msg_stderr\n", tests);
+	++tests;
 	if (!msg_file)
-		++errors, printf("Test3: failed to create msg_file");
+		++errors, printf("Test%d: failed to create msg_file\n", tests);
+	++tests;
 	if (!msg_syslog)
-		++errors, printf("Test4: failed to create msg_syslog");
+		++errors, printf("Test%d: failed to create msg_syslog\n", tests);
+	++tests;
+	if (!msg_filter)
+		++errors, printf("Test%d: failed to create msg_filter\n", tests);
+	++tests;
 	if (!msg_plex)
-		++errors, printf("Test5: failed to create msg_plex");
+		++errors, printf("Test%d: failed to create msg_plex\n", tests);
+	++tests;
 	if (msg_add_plex(msg_plex, msg_file) == -1)
-		++errors, printf("Test6: failed to add msg_file to msg_plex\n");
+		++errors, printf("Test%d: failed to add msg_file to msg_plex\n", tests);
+	++tests;
 	if (msg_add_plex(msg_plex, msg_syslog) == -1)
-		++errors, printf("Test7: failed to add msg_syslog to msg_plex\n");
+		++errors, printf("Test%d: failed to add msg_syslog to msg_plex\n", tests);
+	++tests;
+	if (msg_add_plex(msg_plex, msg_filter) == -1)
+		++errors, printf("Test%d: failed to add msg_filter to msg_plex\n", tests);
 
 	out = dup(STDOUT_FILENO);
 	freopen(msg_stdout_name, "w", stdout);
@@ -1612,9 +1812,16 @@ int main(int ac, char **av)
 	dup2(out, STDOUT_FILENO);
 	close(out);
 
-	errors += verify("Test8", msg_stdout_name, mesg);
-	errors += verify("Test9", msg_stderr_name, mesg);
-	errors += verify("Test10", msg_file_name, mesg);
+	errors += verify(++tests, msg_stdout_name, mesg);
+	errors += verify(++tests, msg_stderr_name, mesg);
+	errors += verify(++tests, msg_file_name, mesg);
+
+	filtered_mesglen = prefix_filter(&filtered_mesg, mesg, strlen(mesg));
+	if (filtered_mesg && filtered_mesglen > 0)
+	{
+		errors += verify(++tests, msg_filter_name, filtered_mesg);
+		mem_destroy(&filtered_mesg);
+	}
 
 	for (i = 0; syslog_facility_map[i].name; ++i)
 	{
@@ -1664,7 +1871,7 @@ int main(int ac, char **av)
 		++errors, printf("Test%d: syslog_parse(\"gibberish\") failed\n", tests);
 
 	if (errors)
-		printf("%d/%d tests failed\n%s\n    %s", errors, 10 + tests, note, mesg);
+		printf("%d/%d tests failed\n%s\n    %s", errors, tests, note, mesg);
 	else
 		printf("All tests passed\n%s\n    %s", note, mesg);
 
