@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 2001 raf <raf@raf.org>
+* Copyright (C) 1999-2002, 2004, 2010, 2020-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-* or visit http://www.gnu.org/copyleft/gpl.html
+* along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20020916 raf <raf@raf.org>
+* 20230313 raf <raf@raf.org>
 */
 
 /*
@@ -88,11 +86,17 @@
  *    added ability to mimic glibc or solaris behaviour
  *    now ISO C 89 compliant formatting except format is not an mbstr
  *    added manpage in pod format (perldoc -F snprintf.c)
- *    added rigorous testing
+ *    added rigorous testing (145893 tests)
  *
  *  raf (raf@raf.org) May 2002
  *    added solaris 8 bug compatibility
  *
+ *  raf (raf@raf.org) Dec 2003
+ *    added examples to the manpage
+ *
+ *  raf (raf@raf.org) Jun 2010
+ *    fixed for 64-bit systems
+ *
  **************************************************************/
 
 /*
@@ -104,14 +108,16 @@ I<snprintf(3)> - safe sprintf
 =head1 SYNOPSIS
 
     #include <slack/std.h>
+    #ifndef HAVE_SNPRINTF
     #include <slack/snprintf.h>
+    #endif
 
     int snprintf(char *str, size_t size, const char *format, ...);
     int vsnprintf(char *str, size_t size, const char *format, va_list args);
 
 =head1 DESCRIPTION
 
-Safe version of I<sprintf(3)> of that doesn't suffer from buffer overruns.
+Safe version of I<sprintf(3)> that doesn't suffer from buffer overruns.
 
 =over 4
 
@@ -190,32 +196,30 @@ int sprintf();
 
 =item C<int snprintf(char *str, size_t size, const char *format, ...)>
 
-Writes output to the string C<str>, under control of the format string
+Writes output to the buffer, C<str>, under control of the format string
 C<format>, that specifies how subsequent arguments are converted for output.
 It is similar to I<sprintf(3)>, except that C<size> specifies the maximum
-number of characters to produce. The trailing C<nul> character is counted
-towards this limit, so you must allocate at least C<size> characters for
-C<str>.
+number of bytes to produce. The trailing C<nul> byte is counted towards this
+limit, so you must allocate at least C<size> bytes for C<str>.
 
-If C<size> is zero, nothing is written and C<str> may be C<null>. Otherwise,
-output characters beyond the C<n-1>st are discarded rather than being
-written to C<str>, and a C<nul> character is written at the end of the
-characters actually written to C<str>. If copying takes place between
-objects that overlap, the behaviour is undefined.
+If C<size> is zero, nothing is written, and C<str> may be C<null>.
+Otherwise, output bytes beyond the C<n-1>st are discarded rather than being
+written to C<str>, and a C<nul> byte is written at the end of the bytes
+actually written to C<str>. If copying takes place between objects that
+overlap, the behaviour is undefined.
 
-On success, returns the number of characters that would have been written
-had C<size> been sufficiently large, not counting the terminating C<nul>
-character. Thus, the C<nul>-terminated output has been completely written if
-and only if the return value is nonnegative and less than C<size>. On error,
-returns C<-1> (i.e. encoding error).
+On success, returns the number of bytes that would have been written had
+C<size> been sufficiently large, not counting the terminating C<nul> byte.
+Thus, the C<nul>-terminated output has been completely written if and only
+if the return value is non-negative and less than C<size>. On error, returns
+C<-1> (i.e. encoding error).
 
-Note that if your system already has I<snprintf(3)> but this implementation
+Note that if your system already has I<snprintf(3)>, but this implementation
 was installed anyway, it's because the system implementation has a broken
 return value. Some older implementations (e.g. I<glibc-2.0>) return C<-1>
-when the string is truncated rather than returning the number of characters
-that would have been written had C<size> been sufficiently large (not
-counting the terminating C<nul> character) as required by I<ISO/IEC
-9899:1999(E)>.
+when the string is truncated rather than returning the number of bytes that
+would have been written had C<size> been sufficiently large (not counting
+the terminating C<nul> byte) as required by I<ISO/IEC 9899:1999(E)>.
 
 =cut
 
@@ -833,7 +837,7 @@ int vsnprintf(char *str, size_t size, const char *format, va_list args)
 						if (cflags == SIZE_SHORT)
 							value = (short)va_arg(args, int);
 						else if (cflags == SIZE_LONG)
-							value = (long)va_arg(args, int);
+							value = (long)va_arg(args, long);
 						else
 							value = (int)va_arg(args, int);
 
@@ -845,7 +849,7 @@ int vsnprintf(char *str, size_t size, const char *format, va_list args)
 						if (cflags == SIZE_SHORT)
 							value = (unsigned short)va_arg(args, int);
 						else if (cflags == SIZE_LONG)
-							value = (unsigned long)va_arg(args, int);
+							value = (unsigned long)va_arg(args, long);
 						else
 							value = (unsigned int)va_arg(args, int);
 
@@ -857,7 +861,7 @@ int vsnprintf(char *str, size_t size, const char *format, va_list args)
 						if (cflags == SIZE_SHORT)
 							value = (unsigned short)va_arg(args, int);
 						else if (cflags == SIZE_LONG)
-							value = (unsigned long)va_arg(args, int);
+							value = (unsigned long)va_arg(args, long);
 						else
 							value = (unsigned int)va_arg(args, int);
 
@@ -871,7 +875,7 @@ int vsnprintf(char *str, size_t size, const char *format, va_list args)
 						if (cflags == SIZE_SHORT)
 							value = (unsigned short)va_arg(args, int);
 						else if (cflags == SIZE_LONG)
-							value = (unsigned long)va_arg(args, int);
+							value = (unsigned long)va_arg(args, long);
 						else
 							value = (unsigned int)va_arg(args, int);
 
@@ -994,44 +998,124 @@ int vsnprintf(char *str, size_t size, const char *format, va_list args)
 
 =head1 MT-Level
 
-MT-Safe - provided that the locale is only set by the main thread before
+I<MT-Safe> - provided that the locale is only set by the main thread before
 starting any other threads.
 
+=head1 EXAMPLES
+
+How long is a piece of string?
+
+    #include <slack/std.h>
+    #ifndef HAVE_SNPRINTF
+    #include <slack/snprintf.h>
+    #endif
+
+    int main(int ac, char **av)
+    {
+        char *str;
+        int len;
+
+        len = snprintf(NULL, 0, "%s %d", *av, ac);
+        printf("this string has length %d\n", len);
+
+        if (!(str = malloc((len + 1) * sizeof(char))))
+            return EXIT_FAILURE;
+
+        len = snprintf(str, len + 1, "%s %d", *av, ac);
+        printf("%s %d\n", str, len);
+
+        free(str);
+
+        return EXIT_SUCCESS;
+    }
+
+Check if truncation occurred:
+
+    #include <slack/std.h>
+    #ifndef HAVE_SNPRINTF
+    #include <slack/snprintf.h>
+    #endif
+
+    int main()
+    {
+        char str[16];
+        int len;
+
+        len = snprintf(str, 16, "%s %d", "hello world", 1000);
+        printf("%s\n", str);
+
+        if (len >= 16)
+            printf("length truncated (from %d)\n", len);
+
+        return EXIT_SUCCESS;
+    }
+
+Allocate memory only when needed to prevent truncation:
+
+    #include <slack/std.h>
+    #ifndef HAVE_SNPRINTF
+    #include <slack/snprintf.h>
+    #endif
+
+    int main(int ac, char **av)
+    {
+        char buf[16];
+        char *str = buf;
+        char *extra = NULL;
+        int len;
+
+        if (!av[1])
+            return EXIT_FAILURE;
+
+        if ((len = snprintf(buf, 16, "%s", av[1])) >= 16)
+            if (extra = malloc((len + 1) * sizeof(char)))
+                snprintf(str = extra, len + 1, "%s", av[1]);
+
+        printf("%s\n", str);
+
+        if (extra)
+            free(extra);
+
+        return EXIT_SUCCESS;
+    }
+
 =head1 BUGS
 
-The format control string, C<format>, should be interpreted as a multibyte
+The format control string, C<format>, should be interpreted as a multi-byte
 character sequence but it is not. Apart from that, these functions comply
-with the ISO C 89 formatting requirements specified for the I<fprintf(3)>
+with the I<ISO C 89> formatting requirements specified for the I<fprintf(3)>
 function (section 7.9.6.1).
 
-Even though I<snprintf(3)> is an ISO C 99 function (section 7.19.6.5), this
-implementation does not support the new ISO C 99 formatting conversions or
-length modifiers (i.e. C<%hh[diouxXn]>, C<%ll[diouxXn]>, C<%j[diouxXn]>,
-C<%z[diouxXn]>, C<%t[diouxXn]>, C<%ls>, C<%lc> and C<%[aAF]>). The main
-reason is that the local system's I<sprintf(3)> function is used to perform
-floating point formatting. If the local system can support C<%[aA]>, then
-you must have C99 already and so you must also have I<snprintf(3)> already.
+Even though I<snprintf(3)> is an I<ISO C 99> function (section 7.19.6.5),
+this implementation does not support the new I<ISO C 99> formatting
+conversions or length modifiers (i.e. C<%hh[diouxXn]>, C<%ll[diouxXn]>,
+C<%j[diouxXn]>, C<%z[diouxXn]>, C<%t[diouxXn]>, C<%ls>, C<%lc> and
+C<%[aAF]>). The main reason is that the local system's I<sprintf(3)>
+function is used to perform floating point formatting. If the local system
+can support C<%[aA]>, then you must have C99 already, and so you must also
+have I<snprintf(3)> already.
 
-If I<snprintf(3)> or I<vsnprintf(3)> require more than 512 bytes of space in
-which to format a floating point number, but fail to allocate the required
-space, the floating point number will not be formatted at all and processing
-will continue. There is no indication to the client that an error occurred.
-The chances of this happening are remote. It would take a field width or
-precision greater than the available memory to trigger this bug. Since there
-are only 15 significant digits in a I<double> and only 18 significant digits
-in an 80 bit I<long double> (33 significant digits in a 128 bit long
-double), a precision larger than 15/18/33 serves no purpose and a field
-width larger than the useful output serves no purpose.
+If I<snprintf(3)> or I<vsnprintf(3)> require more than C<512> bytes of space
+in which to format a floating point number, but fail to allocate the
+required space, the floating point number will not be formatted at all, and
+processing will continue. There is no indication to the client that an error
+occurred. The chances of this happening are remote. It would take a field
+width or precision greater than the available memory to trigger this bug.
+Since there are only C<15> significant digits in a I<double> and only C<18>
+significant digits in an C<80> bit I<long double> (C<33> significant digits
+in a C<128> bit long double), a precision larger than C<15>/C<18>/C<33>
+serves no purpose, and a field width larger than the useful output serves no
+purpose.
 
 =head1 SEE ALSO
 
-L<printf(3)|printf(3)>,
-L<sprintf(3)|sprintf(3)>,
-L<vsprintf(3)|vsprintf(3)>
+I<printf(3)>,
+I<sprintf(3)>,
+I<vsprintf(3)>
 
 =head1 AUTHOR
 
-20020916 raf <raf@raf.org>,
+2002-2023 raf <raf@raf.org>,
 1998 Andrew Tridgell <tridge@samba.org>,
 1998 Michael Elkins <me@cs.hmc.edu>,
 1998 Thomas Roessler <roessler@guug.de>,
@@ -1809,10 +1893,28 @@ int main(int ac, char **av)
 
 	printf("Testing: %s\n", "snprintf");
 
+	/*_
+	** Note that, in the macros below, the assignment of sprintf()'s
+	** return value to a variable called "suppress_optimization" is
+	** is extremely important when using recent versions of gcc.
+	** Without it, gcc would optimize the call to sprintf() into a
+	** call to strcpy() in the case when the format is "%s".
+	** That would crash while the actual sprintf() in glibc would work
+	** as expected and put "(null)" into the buffer. Note that this only
+	** matters when the value is NULL which is only included in the tests
+	** when HAVE_PRINTF_STR_FMT_NULL is defined. Presumably, at some point,
+	** gcc's optimizer will remove the unused assignment and then this
+	** problem will resurface. There's a pretense at a use which seems to
+	** make gcc behave itself for now (but it might get smarter one day
+	** and these tests might break again).
+	*/
+
 #define TEST_SNPRINTF1(format, val, vfmt) \
 	{ \
 		char valstr[1024]; \
-		sprintf(valstr, (vfmt), (val)); \
+		int suppress_optimization = sprintf(valstr, (vfmt), (val)); \
+		if (suppress_optimization) \
+			/* suppress gcc warning */; \
 		++tests; \
 		len1 = snprintf(buf1, sizeof(buf1), (format), (val)); \
 		len2 = sprintf(buf2, (format), (val)); \
@@ -1827,7 +1929,9 @@ int main(int ac, char **av)
 #define TEST_SNPRINTF2(format, width, precision, val, vfmt) \
 	{ \
 		char valstr[1024]; \
-		sprintf(valstr, (vfmt), (val)); \
+		int suppress_optimization = sprintf(valstr, (vfmt), (val)); \
+		if (suppress_optimization) \
+			/* suppress gcc warning */; \
 		++tests; \
 		len1 = snprintf(buf1, sizeof(buf1), (format), (width), (precision), (val)); \
 		len2 = sprintf(buf2, (format), (width), (precision), (val)); \
@@ -1842,7 +1946,9 @@ int main(int ac, char **av)
 #define TEST_SNPRINTF3(format, arg, val, vfmt) \
 	{ \
 		char valstr[1024]; \
-		sprintf(valstr, (vfmt), (val)); \
+		int suppress_optimization = sprintf(valstr, (vfmt), (val)); \
+		if (suppress_optimization) \
+			/* suppress gcc warning */; \
 		++tests; \
 		len1 = snprintf(buf1, sizeof(buf1), (format), (arg), (val)); \
 		len2 = sprintf(buf2, (format), (arg), (val)); \
